# -*- coding: utf-8 -*-
#
# Copyright 2017 Mycroft AI Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
"""
    Parse functions for Italian (IT-IT)

    TODO: numbers greater than 999999
    TODO: it_number_parse
    TODO: it_pruning

"""

from datetime import datetime

from dateutil.relativedelta import relativedelta

from tnalagmes.lang.parse_common import is_numeric, look_for_fractions

# Undefined articles ["un", "una", "un'"] can not be supressed,
# in Italian, "un cavallo" means "a horse" or "one horse".
it_articles = ["il", "lo", "la", "i", "gli", "le"]

it_numbers = {
    "zero": 0,
    "un": 1,
    "uno": 1,
    "una": 1,
    "un'": 1,
    "due": 2,
    "tre": 3,
    "quattro": 4,
    "cinque": 5,
    "sei": 6,
    "sette": 7,
    "otto": 8,
    "nove": 9,
    "dieci": 10,
    "undici": 11,
    "dodici": 12,
    "tredici": 13,
    "quattordici": 14,
    "quindici": 15,
    "sedici": 16,
    "diciassette": 17,
    "diciotto": 18,
    "diciannove": 19,
    "venti": 20,
    "vent": 20,
    "trenta": 30,
    "trent": 30,
    "quaranta": 40,
    "quarant": 40,
    "cinquanta": 50,
    "cinquant": 50,
    "sessanta": 60,
    "sessant": 60,
    "settanta": 70,
    "settant": 70,
    "ottanta": 80,
    "ottant": 80,
    "novanta": 90,
    "novant": 90,
    "cento": 100,
    "duecento": 200,
    "trecento": 300,
    "quattrocento": 400,
    "cinquecento": 500,
    "seicento": 600,
    "settecento": 700,
    "ottocento": 800,
    "novecento": 900,
    "primo": 1,
    "secondo": 2,
    "mille": 1000,
    "mila": 1000
}


def isFractional_it(input_str):
    """
    This function takes the given text and checks if it is a fraction.
    E' la versione portoghese riadattata in italiano

    Args:
        text (str): the string to check if fractional
    Returns:
        (bool) or (float): False if not a fraction, otherwise the fraction

    TODO:  verificare la corretta gestione dei plurali
    """

    aFrac = ["mezz", "terz", "quart", "quint", "sest", "settim", "ottav",
             "non", "decim", "undicesim", "dodicesim", "tredicesim",
             "quattrodicesim", "quindicesim", "sedicesim",
             "diciasettesim", "diciottesim", "diciasettesim",
             "diciannovesim"]

    if input_str[:-1].lower() in aFrac:
        return 1.0 / (aFrac.index(input_str[:-1]) + 2)
    if input_str[:-1] == "ventesim":
        return 1.0 / 20
    if input_str[:-1] == "centesim":
        return 1.0 / 100
    if input_str[:-1] == "millesim":
        return 1.0 / 1000

    return False


def extractnumber_long_it(word):
    """
    Questa funzione converte un numero testuale lungo es.
    ventisette -> 27
    quarantuno -> 41
    nell'equivalente valore intero
     args:
         text (str): la stringa da normalizzare
    Ritorna:
         (int) : il valore del numero estratto usando tutta la parola
         Falso : se la parola non è un numero es."qualcuno"
    """
    result = False
    value = False

    for number in it_numbers.keys():  # ciclo unità
        if word.endswith(number):
            result = True
            value = it_numbers[number]
            word = word[0: len(word) - len(number)]
            break

    if result:  # tolte le unità, dovrebbe rimanere una stringa nota
        if word in it_numbers:
            value += it_numbers[word]
        else:
            value = False  # non è un numero es. qualcuno

    return value


def extractnumber_it(text):
    """
    Questa funzione prepara il testo dato per l'analisi rendendo
    numeri testuali come interi o frazioni.
    In italiano non è un modo abituale ma può essere interessante
    per Mycroft
    E' la versione portoghese riadattata in italiano
     args:
         text (str): la stringa da normalizzare
    Ritorna:
         (int) o (float): il valore del numero estratto

    """
    aWords = text.split()
    count = 0
    result = None
    while count < len(aWords):
        val = 0
        word = aWords[count]
        next_next_word = None
        if count + 1 < len(aWords):
            next_word = aWords[count + 1]
            if count + 2 < len(aWords):
                next_next_word = aWords[count + 2]
        else:
            next_word = None

        # is current word a number?
        if word in it_numbers:
            if word == "mila":
                val = it_numbers[word]
                val = result * val
                result = 0
            else:
                val = it_numbers[word]

        elif word.isdigit():  # doesn't work with decimals
            val = int(word)
        elif is_numeric(word):
            val = float(word)

        elif isFractional_it(word):
            if not result:
                result = 1
            result = result * isFractional_it(word)
            # "un terzo" is 1/3 but "il terzo" is 3
            if aWords[count - 1] == "il":
                result = 1.0 // isFractional_it(word)

            count += 1
            continue

        if not val:
            # look for fractions like "2/3"
            aPieces = word.split('/')
            # if (len(aPieces) == 2 and is_numeric(aPieces[0])
            #   and is_numeric(aPieces[1])):
            if look_for_fractions(aPieces):
                val = float(aPieces[0]) / float(aPieces[1])

        if not val:
            # cerca numero composto come ventuno ventitre centoventi"
            val = extractnumber_long_it(word)

        if val:
            if result is None:
                result = 0
            # handle fractions
            # if next_word != "avos":
            result += val
            # else:
            #    result = float(result) / float(val)

        if next_word is None:
            break

        # number word and fraction
        ands = ["e"]
        if next_word in ands:
            zeros = 0
            if result is None:
                count += 1
                continue
            newWords = aWords[count + 2:]
            newText = ""
            for word in newWords:
                newText += word + " "

            afterAndVal = extractnumber_it(newText[:-1])
            if afterAndVal:
                if result < afterAndVal or result < 20:
                    while afterAndVal > 1:
                        afterAndVal = afterAndVal / 10.0
                    for word in newWords:
                        if word == "zero" or word == "0":
                            zeros += 1
                        else:
                            break
                for _ in range(0, zeros):
                    afterAndVal = afterAndVal / 10.0
                result += afterAndVal
                break
        elif next_next_word is not None:
            if next_next_word in ands:
                newWords = aWords[count + 3:]
                newText = ""
                for word in newWords:
                    newText += word + " "
                afterAndVal = extractnumber_it(newText[:-1])
                if afterAndVal:
                    if result is None:
                        result = 0
                    result += afterAndVal
                    break

        decimals = ["punto", "virgola", ".", ","]
        if next_word in decimals:
            zeros = 0
            newWords = aWords[count + 2:]
            newText = ""
            for word in newWords:
                newText += word + " "
            for word in newWords:
                if word == "zero" or word == "0":
                    zeros += 1
                else:
                    break
            afterDotVal = str(extractnumber_it(newText[:-1]))
            afterDotVal = zeros * "0" + afterDotVal
            result = float(str(result) + "." + afterDotVal)
            break
        count += 1

    if result is None:
        return False

    # Return the $str with the number related words removed
    # (now empty strings, so strlen == 0)
    # aWords = [word for word in aWords if len(word) > 0]
    # text = ' '.join(aWords)
    if "." in str(result):
        integer, dec = str(result).split(".")
        # cast float to int
        if dec == "0":
            result = int(integer)

    return result


def normalize_it(text, remove_articles):
    """ IT string normalization """

    words = text.split()  # this also removed extra spaces
    # Contractions are not common in IT
    # Convert numbers into digits, e.g. "quarantadue" -> "42"
    normalized = ""
    i = 0

    while i < len(words):
        word = words[i]
        # remove articles
        # Italian requires the article to define the gender
        if remove_articles and word in it_articles:
            i += 1
            continue

        if word in it_numbers:
            word = str(it_numbers[word])

        val = extractnumber_long_it(word)

        if val:
            word = str(val)

        normalized += " " + word
        i += 1
    # indefinite articles in it-it can not be removed

    return normalized[1:]


def extract_datetime_it(string, currentDate):
    def clean_string(s):
        """
            cleans the input string of unneeded punctuation and capitalization
            among other things.
            Normalize italian plurals
        """
        symbols = [".", ",", ";", "?", "!", u"º", u"ª", u"°"]

        for word in symbols:
            s = s.replace(word, "")

        s = s.lower().replace(
            u"á",
            "a").replace(
            u"à",
            "a").replace(
            u"è",
            "e'").replace(
            u"é",
            "e'").replace(
            u"ì",
            "i").replace(
            u"ù",
            "u").replace(
            u"ò",
            "o").replace(
            "-",
            " ").replace(
            "_",
            "")

        noise_words = ["tra", "la", "del", "al", "il", "di",
                       "le", "per", "alle", "alla", "dai", "delle",
                       "a", "e'", "era", "questa", "questo", "e"]

        for word in noise_words:
            s = s.replace(" " + word + " ", " ")

        # normalizza plurali per semplificare analisi
        s = s.replace(
            "secondi",
            "secondo").replace(
            "minuti",
            "minuto").replace(
            "ore",
            "ora").replace(
            "giorni",
            "giorno").replace(
            "settimane",
            "settimana").replace(
            "mesi",
            "mese").replace(
            "anni",
            "anno").replace(
            "mattino",
            "mattina").replace(
            "prossima",
            "prossimo").replace(
            "questa",
            "questo").replace(
            "quarti",
            "quarto")

        wordList = s.split()
        # print(wordList)  # debug only

        return wordList

    def date_found():
        return found or \
               (
                       datestr != "" or timeStr != "" or
                       yearOffset != 0 or monthOffset != 0 or
                       dayOffset is True or hrOffset != 0 or
                       hrAbs != 0 or minOffset != 0 or
                       minAbs != 0 or secOffset != 0
               )

    if string == "" or not currentDate:
        return None

    found = False
    daySpecified = False
    dayOffset = False
    monthOffset = 0
    yearOffset = 0
    dateNow = currentDate
    today = dateNow.strftime("%w")
    currentYear = dateNow.strftime("%Y")
    fromFlag = False
    datestr = ""
    hasYear = False
    timeQualifier = ""

    timeQualifiersList = ['mattina', 'pomeriggio', 'sera']
    markers = ['alle', 'in', 'questo', 'per', 'di']
    days = ['lunedi', 'martedi', 'mercoledi',
            'giovedi', 'venerdi', 'sabato', 'domenica']
    months = ['gennaio', 'febbraio', 'marzo', 'aprile', 'maggio', 'giugno',
              'luglio', 'agosto', 'settembre', 'ottobre', 'novembre',
              'dicembre']
    monthsShort = ['gen', 'feb', 'mar', 'apr', 'mag', 'giu', 'lug', 'ago',
                   'set', 'ott', 'nov', 'dic']

    words = clean_string(string)

    for idx, word in enumerate(words):
        if word == "":
            continue
        wordPrevPrev = words[idx - 2] if idx > 1 else ""
        wordPrev = words[idx - 1] if idx > 0 else ""
        wordNext = words[idx + 1] if idx + 1 < len(words) else ""
        wordNextNext = words[idx + 2] if idx + 2 < len(words) else ""
        # wordNextNextNext = words[idx + 3] if idx + 3 < len(words) else ""
        # possono esistere casi dove servano tre parole di profondità ?
        start = idx
        used = 0
        # save timequalifier for later
        if word in timeQualifiersList:
            timeQualifier = word
            # parse today, tomorrow, day after tomorrow
        elif word == "oggi" and not fromFlag:
            dayOffset = 0
            used += 1
        elif word == "domani" and not fromFlag:
            dayOffset = 1
            used += 1
        elif word == "ieri" and not fromFlag:
            dayOffset -= 1
            used += 1
        elif word == "dopodomani" and not fromFlag:  # after tomorrow
            dayOffset += 2
            used += 1
        elif word == "dopo" and wordNext == "domani" and \
                not fromFlag:
            dayOffset += 1
            used += 2
        elif word == "giorno":
            if wordPrev[0].isdigit():
                dayOffset += int(wordPrev)
                start -= 1
                used = 2
                if wordNext == "dopo" and wordNextNext == "domani":
                    dayOffset += 1
                    used += 2
        elif word == "settimana" and not fromFlag:
            if wordPrev[0].isdigit():
                dayOffset += int(wordPrev) * 7
                start -= 1
                used = 2
            elif wordPrev == "prossimo":
                dayOffset = 7
                start -= 1
                used = 2
            elif wordPrev == "passato":
                dayOffset = -7
                start -= 1
                used = 2
                # parse 10 months, next month, last month
        elif word == "mese" and not fromFlag:
            if wordPrev[0].isdigit():
                monthOffset = int(wordPrev)
                start -= 1
                used = 2
            elif wordPrev == "prossimo":
                monthOffset = 1
                start -= 1
                used = 2
            elif wordPrev == "passato":
                monthOffset = -1
                start -= 1
                used = 2
                # parse 5 years, next year, last year
        elif word == "anno" and not fromFlag:
            if wordPrev[0].isdigit():
                yearOffset = int(wordPrev)
                start -= 1
                used = 2
            elif wordPrev == "prossimo":
                yearOffset = 1
                start -= 1
                used = 2
            elif wordPrev == "passato":
                yearOffset = -1
                start -= 1
                used = 2
                # parse Monday, Tuesday, etc., and next Monday,
                # last Tuesday, etc.
        elif word in days and not fromFlag:
            d = days.index(word)
            dayOffset = (d + 1) - int(today)
            used = 1
            if dayOffset < 0:
                dayOffset += 7
            if wordPrev == "prossimo":
                dayOffset += 7
                used += 1
                start -= 1
            elif wordPrev == "passato":
                dayOffset -= 7
                used += 1
                start -= 1
            if wordNext == "prossimo":
                # dayOffset += 7
                used += 1
            elif wordNext == "passato":
                # dayOffset -= 7
                used += 1
                # parse 15 of July, June 20th, Feb 18, 19 of February
        elif word in months or word in monthsShort and not fromFlag:
            try:
                m = months.index(word)
            except ValueError:
                m = monthsShort.index(word)
            used += 1
            datestr = months[m]
            if wordPrev and (wordPrev[0].isdigit()):
                datestr += " " + wordPrev
                start -= 1
                used += 1
                if wordNext and wordNext[0].isdigit():
                    datestr += " " + wordNext
                    used += 1
                    hasYear = True
                else:
                    hasYear = False

            elif wordNext and wordNext[0].isdigit():
                datestr += " " + wordNext
                used += 1
                if wordNextNext and wordNextNext[0].isdigit():
                    datestr += " " + wordNextNext
                    used += 1
                    hasYear = True
                else:
                    hasYear = False
        # parse 5 days from tomorrow, 10 weeks from next thursday,
        # 2 months from July
        validFollowups = days + months + monthsShort
        validFollowups.append("oggi")
        validFollowups.append("domani")
        validFollowups.append("prossimo")
        validFollowups.append("passato")
        validFollowups.append("ora")
        if (word == "da" or word == "dopo") and wordNext in validFollowups:
            used = 2
            fromFlag = True
            if wordNext == "domani":
                dayOffset += 1
            elif wordNext in days:
                d = days.index(wordNext)
                tmpOffset = (d + 1) - int(today)
                used = 2
                if tmpOffset < 0:
                    tmpOffset += 7
                dayOffset += tmpOffset
            elif wordNextNext and wordNextNext in days:
                d = days.index(wordNextNext)
                tmpOffset = (d + 1) - int(today)
                used = 3
                if wordNext == "prossimo":
                    tmpOffset += 7
                    used += 2  # era 1
                    start -= 1
                elif wordNext == "passato":
                    tmpOffset -= 7
                    used += 1
                    start -= 1
                dayOffset += tmpOffset
        if used > 0:
            if start - 1 > 0 and words[start - 1] == "questo":
                start -= 1
                used += 1

            for i in range(0, used):
                words[i + start] = ""

            if start - 1 >= 0 and words[start - 1] in markers:
                words[start - 1] = ""
            found = True
            daySpecified = True

    # parse time
    timeStr = ""
    hrOffset = 0
    minOffset = 0
    secOffset = 0
    hrAbs = 0
    minAbs = 0

    for idx, word in enumerate(words):
        if word == "":
            continue

        wordPrevPrev = words[idx - 2] if idx > 1 else ""
        wordPrev = words[idx - 1] if idx > 0 else ""
        wordNext = words[idx + 1] if idx + 1 < len(words) else ""
        wordNextNext = words[idx + 2] if idx + 2 < len(words) else ""
        # wordNextNextNext = words[idx + 3] if idx + 3 < len(words) else ""
        # TODO verfica se esistono casi dove serva profindita 3 x analisi
        # parse noon, midnight, morning, afternoon, evening
        used = 0
        if word == "mezzogiorno":
            hrAbs = 12
            used += 1
        elif word == "mezzanotte":
            hrAbs = 24
            used += 1
        if word == "mezzo" and wordNext == "giorno":  # if stt splits the word
            hrAbs = 12
            used += 2
        elif word == "mezza" and wordNext == "notte":  # if stt splits the word
            hrAbs = 24
            used += 2
        elif word == "mattina":
            if hrAbs == 0:
                hrAbs = 8
            used += 1
            if wordNext and wordNext[0].isdigit():  # mattina alle 5
                hrAbs = int(wordNext)
                used += 1
        elif word == "pomeriggio":
            if hrAbs == 0:
                hrAbs = 15
            used += 1
            if wordNext and wordNext[0].isdigit():  # pomeriggio alle 5
                hrAbs = int(wordNext)
                used += 1
                if hrAbs < 12:
                    hrAbs += 12
        elif word == "sera":
            if hrAbs == 0:
                hrAbs = 19
            used += 1
            if wordNext and wordNext[0].isdigit():  # sera alle 8
                hrAbs = int(wordNext)
                used += 1
                if hrAbs < 12:
                    hrAbs += 12

        # parse 5:00 am, 12:00 p.m., etc
        elif word[0].isdigit():
            isTime = True
            strHH = ""
            strMM = ""
            remainder = ""
            if ':' in word:
                # parse colons
                # "3:00 in the morning"
                stage = 0
                length = len(word)
                for i in range(length):
                    if stage == 0:
                        if word[i].isdigit():
                            strHH += word[i]

                        elif word[i] == ":":
                            stage = 1
                        else:
                            stage = 2
                            i -= 1
                    elif stage == 1:
                        if word[i].isdigit():
                            strMM += word[i]
                        else:
                            stage = 2
                            i -= 1
                    elif stage == 2:
                        remainder = word[i:].replace(".", "")
                        break

                if remainder == "":
                    nextWord = wordNext.replace(".", "")
                    if nextWord == "am" or nextWord == "pm":
                        remainder = nextWord
                        used += 1
                    elif nextWord == "sera":
                        remainder = "pm"
                        used += 1
                    elif wordNext == "mattina":
                        remainder = "am"
                        used += 1
                    elif wordNext == "pomeriggio":
                        remainder = "pm"
                        used += 1
                    elif wordNext == "notte":
                        remainder = "pm"
                        used += 1
                    elif wordNext == "di" and wordNextNext == "notte":
                        if strHH > 5:
                            remainder = "pm"
                        else:
                            remainder = "am"
                        used += 2
                    else:
                        if timeQualifier != "":
                            if strHH <= 12 and \
                                    (timeQualifier == "sera" or
                                     timeQualifier == "pomeriggio"):
                                strHH += 12

            else:
                # try to parse # s without colons
                # 5 hours, 10 minutes etc.
                length = len(word)
                strNum = ""
                remainder = ""
                for i in range(length):
                    if word[i].isdigit():
                        strNum += word[i]
                    else:
                        remainder += word[i]

                if remainder == "":
                    remainder = wordNext.replace(".", "").lstrip().rstrip()

                if (
                        remainder == "pm" or
                        wordNext == "pm" or
                        remainder == "p.m." or
                        wordNext == "p.m."):
                    strHH = strNum
                    remainder = "pm"
                    used = 1
                elif (
                        remainder == "am" or
                        wordNext == "am" or
                        remainder == "a.m." or
                        wordNext == "a.m."):
                    strHH = strNum
                    remainder = "am"
                    used = 1
                else:
                    if wordNext == "pm" or wordNext == "p.m.":
                        strHH = strNum
                        remainder = "pm"
                        used = 1
                    elif wordNext == "am" or wordNext == "a.m.":
                        strHH = strNum
                        remainder = "am"
                        used = 1
                    elif (
                            int(word) > 100 and
                            (
                                    wordPrev == "o" or
                                    wordPrev == "oh"
                            )):
                        # 0800 hours (pronounced oh-eight-hundred)
                        strHH = int(word) / 100
                        strMM = int(word) - strHH * 100
                        if wordNext == "ora":
                            used += 1

                    elif (
                            wordNext == "ora" and
                            word[0] != '0' and
                            (
                                    int(word) < 100 and
                                    int(word) > 2400
                            )):
                        # ignores military time
                        # "in 3 hours"
                        hrOffset = int(word)
                        used = 2
                        isTime = False
                        hrAbs = -1
                        minAbs = -1
                    elif wordNext == "mattina":
                        # " 11 del mattina"  -> del viene rimosso
                        hh = int(word)
                        used = 2
                        isTime = False
                        hrAbs = hh
                        minAbs = 00
                    elif wordNext == "pomeriggio":
                        # " 2 del pomeriggio"  -> del viene rimosso
                        hh = int(word)
                        if hh < 12:
                            hh += 12
                        used = 2
                        isTime = False
                        hrAbs = hh
                        minAbs = 00
                    elif wordNext == "sera":
                        # "alle 8 di sera"  -> alle viene rimosso
                        hh = int(word)
                        if hh < 12:
                            hh += 12
                        used = 2
                        isTime = False
                        hrAbs = hh
                        minAbs = 00
                    # parse half an hour : undici e mezza
                    elif wordNext and wordNext == "mezza":
                        hrAbs = int(word)
                        minAbs = 30
                        used = 2
                        isTime = False
                    # parse 1 quarter hour 3 quarters : dieci e tre quarti
                    elif word and wordNext and \
                            wordNext == "quarto" and word[0].isdigit():
                        minAbs = 15 * int(word)
                        used = 2
                        if minAbs > 45:  # elimina eventuali errori
                            minAbs = 0
                            used -= 2
                        isTime = False
                    elif wordNext == "minuto":
                        # "in 10 minutes"
                        minOffset = int(word)
                        used = 2
                        isTime = False
                        hrAbs = -1
                        minAbs = -1
                    elif wordNext == "secondo":
                        # in 5 seconds
                        secOffset = int(word)
                        used = 2
                        isTime = False
                        hrAbs = -1
                        minAbs = -1
                    elif int(word) > 100:
                        strHH = int(word) / 100
                        strMM = int(word) - strHH * 100
                        if wordNext == "ora":
                            used += 1
                    elif wordNext and wordNext[0].isdigit():
                        strHH = word
                        strMM = wordNext
                        used += 1
                        if wordNextNext == "ora":
                            used += 1
                    elif wordNext == "in" and wordNextNext == "punto":
                        strHH = word
                        strMM = 00
                        used += 2

                    else:
                        isTime = False

            strHH = int(strHH) if strHH else 0
            strMM = int(strMM) if strMM else 0
            strHH = strHH + 12 if remainder == "pm" and strHH < 12 else strHH
            strHH = strHH - 12 if remainder == "am" and strHH >= 12 else strHH
            if strHH > 24 or strMM > 59:
                isTime = False
                used = 0
            if isTime:
                hrAbs = strHH * 1
                minAbs = strMM * 1
                used += 1

            if hrAbs <= 12 and (timeQualifier == "sera" or
                                timeQualifier == "pomeriggio"):
                hrAbs += 12

        if used > 0:
            # removed parsed words from the sentence
            for i in range(used):
                words[idx + i] = ""

            if wordPrev == "o" or wordPrev == "oh":
                words[words.index(wordPrev)] = ""

            if wordPrev == "presto":
                hrOffset = -1
                words[idx - 1] = ""
                idx -= 1
            elif wordPrev == "tardi":
                hrOffset = 1
                words[idx - 1] = ""
                idx -= 1
            if idx > 0 and wordPrev in markers:
                words[idx - 1] = ""
            if idx > 1 and wordPrevPrev in markers:
                words[idx - 2] = ""

            idx += used - 1
            found = True

    # check that we found a date
    if not date_found:
        return None

    if dayOffset is False:
        dayOffset = 0

    # perform date manipulation

    extractedDate = dateNow
    extractedDate = extractedDate.replace(microsecond=0,
                                          second=0,
                                          minute=0,
                                          hour=0)

    if datestr != "":
        en_months = ['january', 'february', 'march', 'april', 'may', 'june',
                     'july', 'august', 'september', 'october', 'november',
                     'december']
        en_monthsShort = ['jan', 'feb', 'mar', 'apr', 'may', 'june', 'july',
                          'aug',
                          'sept', 'oct', 'nov', 'dec']
        for idx, en_month in enumerate(en_months):
            datestr = datestr.replace(months[idx], en_month)
        for idx, en_month in enumerate(en_monthsShort):
            datestr = datestr.replace(monthsShort[idx], en_month)

        temp = datetime.strptime(datestr, "%B %d")
        if not hasYear:
            temp = temp.replace(year=extractedDate.year)
            if extractedDate < temp:
                extractedDate = extractedDate.replace(year=int(currentYear),
                                                      month=int(
                                                          temp.strftime(
                                                              "%m")),
                                                      day=int(temp.strftime(
                                                          "%d")))
            else:
                extractedDate = extractedDate.replace(
                    year=int(currentYear) + 1,
                    month=int(temp.strftime("%m")),
                    day=int(temp.strftime("%d")))
        else:
            extractedDate = extractedDate.replace(
                year=int(temp.strftime("%Y")),
                month=int(temp.strftime("%m")),
                day=int(temp.strftime("%d")))

    if timeStr != "":
        temp = datetime(timeStr)
        extractedDate = extractedDate.replace(hour=temp.strftime("%H"),
                                              minute=temp.strftime("%M"),
                                              second=temp.strftime("%S"))

    if yearOffset != 0:
        extractedDate = extractedDate + relativedelta(years=yearOffset)
    if monthOffset != 0:
        extractedDate = extractedDate + relativedelta(months=monthOffset)
    if dayOffset != 0:
        extractedDate = extractedDate + relativedelta(days=dayOffset)
    if hrAbs != -1 and minAbs != -1:

        extractedDate = extractedDate + relativedelta(hours=hrAbs,
                                                      minutes=minAbs)
        if (hrAbs != 0 or minAbs != 0) and datestr == "":
            if not daySpecified and dateNow > extractedDate:
                extractedDate = extractedDate + relativedelta(days=1)
    if hrOffset != 0:
        extractedDate = extractedDate + relativedelta(hours=hrOffset)
    if minOffset != 0:
        extractedDate = extractedDate + relativedelta(minutes=minOffset)
    if secOffset != 0:
        extractedDate = extractedDate + relativedelta(seconds=secOffset)
    for idx, word in enumerate(words):
        if words[idx] == "e" and words[idx - 1] == "" and words[
            idx + 1] == "":
            words[idx] = ""

    resultStr = " ".join(words)
    resultStr = ' '.join(resultStr.split())
    return [extractedDate, resultStr]


def get_gender_it(word, raw_string=""):
    """
    Questa potrebbe non essere utile.
    In italiano per definire il genere è necessario
    analizzare l'articolo che la precede e non la lettera
    con cui finisce la parola, ma sono presenti funzioni per
    la rimozione degli articoli dalla frase per semplificarne
    l'analisi

    TODO:  verificare se utile
    """

    gender = False
    words = raw_string.split(" ")
    for idx, w in enumerate(words):
        if w == word and idx != 0:
            previous = words[idx - 1]
            gender = get_gender_it(previous)
            break

    if not gender:
        if word[-1] == "a" or word[-1] == "e":
            gender = "f"
        if word[-1] == "o" or word[-1] == "n" \
                or word[-1] == "l" or word[-1] == "i":
            gender = "m"

    return gender



TERMINOLOGY = {
    'ammunition': ['ammunition'],
    'armour': ['clothing'],
    'attack_damage': ['injuries'],
    'enemy': ['riders'],
    'fuel': ['your oxen team'],
    'medicine': ['miscellaneous supplies'],
    'passive_damage': ['pneumonia'],
    'supplies': ['food'],
    "currency": ["cash"]
}

RANDOM_EVENTS = {
    'attack': {'conclusion': 'quickest draw outside of dodge city!!!\n'
                             "you got 'em!",
               'damage': 'you got shot in the leg and they took one '
                         'of your oxen\n'
                         'better have a doc look at your wound',
               'die': '',
               'error': 'you ran out of bullets---they get lots of '
                        'cash',
               'events': [],
               'intro': 'bandits attack'
               },
    'illness': {'conclusion': '',
                'damage': '',
                'die': '',
                'error': '',
                'events': [],
                'intro': ''
                },
    'poison': {'conclusion': '',
               'die': 'you die of snakebite since you have no '
                      'medicine',
               'error': '',
               'events': [],
               'intro': 'you killed a poisonous snake after it bit '
                        'you'
               },
    'animal': {'conclusion': "nice shootin' pardner---they didn't get "
                             'much',
               'damage': 'you were too low on bullets--\n'
                         'the wolves overpowered you',
               'die': '',
               'error': 'slow on the draw---they got at your food and '
                        'clothes',
               'events': [],
               'intro': 'wild animals attack!'
               },
    'companion': {'conclusion': '',
                  'damage': '',
                  'die': '',
                  'error': '',
                  'events': [],
                  'intro': 'bad luck---your daughter broke her arm\n'
                           'you had to stop and use supplies to make '
                           'a sling'
                  },
    'bad_terrain': {'conclusion': '',
                    'damage': '',
                    'die': '',
                    'error': '',
                    'events': [],
                    'intro': 'wagon gets swamped fording river--lose '
                             'food and clothes'},
    'companion_lose': {'conclusion': '',
                       'damage': '',
                       'die': '',
                       'error': '',
                       'events': [],
                       'intro': 'your son gets lost---spend half the '
                                'day looking for him'},
    'find_supplies': {'conclusion': '',
                      'damage': '',
                      'die': '',
                      'error': '',
                      'events': [],
                      'intro': 'helpful indians show you where to '
                               'find more food'},
    'fuel_damage': {'conclusion': '',
                    'damage': '',
                    'die': '',
                    'error': '',
                    'events': [],
                    'intro': 'ox wanders off---spend time looking for '
                             'it'
                    },
    'shelter_damage': {'conclusion': '',
                       'damage': '',
                       'die': '',
                       'error': '',
                       'events': [],
                       'intro': 'wagon breaks down--lose time and '
                                'supplies fixing it'
                       },
    'shelter_fire': {'conclusion': '',
                     'damage': '',
                     'die': '',
                     'error': '',
                     'events': [],
                     'intro': 'there was a fire in your wagon--food '
                              'and supplies damaged'
                     },
    'supply_damage': {'conclusion': '',
                      'damage': '',
                      'die': '',
                      'error': '',
                      'events': [],
                      'intro': 'unsafe water--lose time looking for '
                               'clean spring'
                      },
    'vehicle_damage': {'conclusion': '',
                       'damage': '',
                       'die': '',
                       'error': '',
                       'events': [],
                       'intro': 'ox injures leg---slows you down rest '
                                'of trip'
                       },
    'heavy_rain': {'conclusion': '',
                   'die': '',
                   'error': '',
                   'events': [],
                   'intro': 'heavy rains---time and supplies lost'
                   },
    'storm': {'conclusion': '',
              'die': '',
              'error': '',
              'events': [],
              'intro': 'hail storm---supplies damaged'
              },
    'fog': {'conclusion': '',
            'die': '',
            'error': '',
            'events': [],
            'intro': 'lose your way in heavy fog---time is '
                     'lost'
            },
    'cold': {'conclusion': 'you have enough clothing to keep '
                           'you warm',
             'die': '',
             'error': "you don't have enough clothing to "
                      'keep you warm',
             'events': [],
             'intro': 'cold weather---brrrrrrr!---'
             }
}

GAME_EVENTS = {
    'damage': {'conclusion': '',
               'error': 'you ran out medical supplies\nyou died of',
               'high': 'bad illness---medicine used',
               'intro': '',
               'mild': 'mild illness---medicine used'},
    'difficulty_damage': {'conclusion': '',
                          'error': '',
                          'events': [],
                          'intro': 'blizzard in mountain pass--time and supplies '
                                   'lost'},
    'easy_difficulty': {'conclusion': 'you made it safely through south pass--no '
                                      'snow',
                        'events': ['you got lost---lose valuable time trying to '
                                   'find trail!',
                                   'wagon damaged!---lose time and supplies',
                                   'the going gets slow'],
                        'intro': 'rugged mountains'
                        },
    'enemy_encounter': {'conclusion': '',
                        'damage': 'lousy shot---you got knifed\n'
                                  "you have to see ol' doc blanchard",
                        'die': 'you ran out of bullets and got massacred by the ',
                        'enemy_run': 'they did not attack',
                        'high_score': 'nice shooting---you drove them off',
                        'hostile_conclusion': ' were hostile--check for losses',
                        'hostile_intro': 'they look hostile',
                        'intro': ' ahead.',
                        'low_score': 'kinda slow with your colt .45',
                        'number_questions': ['tactics\n'
                                             '(1) run  (2) attack  (3) continue  '
                                             '(4) circle wagons\n'
                                             "if you run you'll gain time but "
                                             'wear down your oxen\n'
                                             "if you circle you'll lose time"],
                        'peaceful_conclusion': ' were friendly, but check for '
                                               'possible losses',
                        'peaceful_intro': "they don't look hostile"
                        },
    'explore': {'conclusion': '',
                'error': 'tough---you need more bullets to go hunting',
                'events': ["ri\x07ght betwee\x07n the eye\x07's---you got a\x07 "
                           'big one!!\x07!!',
                           'sorry---no luck today',
                           'nice shot--right through the neck--feast tonight!!'],
                'intro': ''
                },
    'game_over': {'conclusion': 'distance remaining: ',
                  'error': 'time has run out.  winter has set in and you did not '
                           'reach oregon.'
                  },
    'hard_difficulty': {'conclusion': '', 'intro': ''},
    'heal': {'conclusion': "doctor's bill is $20",
             'die': 'you died of ',
             'error': "you can't afford a doctor",
             'events': [],
             'intro': ''
             },
    'intro': {'conclusion': 'you can spend all your money before you start your '
                            'trip -\n'
                            'or you can save some of your cash to spend at forts '
                            'along\n'
                            'the way when you run low.  however, items cost more '
                            'at\n'
                            'the forts.  you can also go hunting along the way to '
                            'get\n'
                            'more food.\n'
                            'whenever you have to use your trusty rifle along the '
                            'way,\n'
                            'you will see the words: type bang.  the faster you '
                            'type\n'
                            "in the word 'bang' and hit the 'return' key, the "
                            'better\n'
                            "luck you'll have with your gun.\n"
                            "when asked to enter money amounts, don't use a '$'.\n"
                            'good luck!!!',
              'intro': 'instructions!\n'
                       'this program simulates a trip over the oregon trail from\n'
                       'independence, missouri to oregon city, oregon in 1847.\n'
                       'your family of five will cover the 2000 mile oregon '
                       'trail\n'
                       'in 5-6 months --- if you make it alive.\n'
                       "you had saved $900 to spend for the trip, and you've "
                       'just\n'
                       '   paid $200 for a wagon.\n'
                       'you will need to spend the rest of your money on the\n'
                       '   following items:\n'
                       '     oxen - you can spend $200-$300 on your team\n'
                       "            the more you spend, the faster you'll go\n"
                       "               because you'll have better animals\n"
                       '     food - the more you have, the less chance there\n'
                       '               is of getting sick\n'
                       '     ammunition - $1 buys a belt of 50 bullets\n'
                       '            you will need bullets for attacks by animals\n'
                       '               and bandits, and for hunting food\n'
                       '     clothing - this is especially important for the '
                       'cold\n'
                       '               weather you will encounter when crossing\n'
                       '               the mountains\n'
                       '     miscellaneous supplies - this includes medicine and\n'
                       '               other things you will need for sickness\n'
                       '               and emergency repairs'
              },
    'inventory': {'conclusion': 'after all your purchases, you now have '
                                '{inv.money} dollars left',
                  'error': 'you overspent--you only had $700 to spend.  buy again',
                  'intro': 'how much do you want to spend on '
                  },
    'lose': {'conclusion': 'we thank you for this information and we are sorry '
                           'you\n'
                           "didn't make it to the great territory of oregon\n"
                           'better luck next time\n'
                           '                              sincerely\n'
                           '                 the oregon city chamber of commerce',
             'error': 'your aunt nellie in st. louis is anxious to hear',
             'intro': 'do to your unfortunate situation, there are a few\n'
                      ' formalities we must go through',
             'positive_response': '',
             'yes_no_questions': ['would you like a minister?',
                                  'would you like a fancy funeral?',
                                  'would you like us to inform your next of '
                                  'kin?']
             },
    'low_supplies': {'intro': "you'd better do some hunting or buy food and "
                              'soon!!!!'
                     },
    'maintenance': {'conclusion': '',
                    'error': "you can't eat that well",
                    'intro': 'do you want to eat (1) poorly  (2) moderately\n'
                             'or (3) well'
                    },
    'medium_difficulty': {'conclusion': 'you made it safely through south '
                                        'pass--no snow',
                          'intro': ''
                          },
    'numeric': {'error': 'impossible', 'high': 'too high', 'low': 'too low'},
    'shop': {'conclusion': '',
             'error': "you don't have that much--keep your spending down",
             'events': [],
             'intro': 'enter what you wish to spend on the following'
             },
    'turn': {'die': 'you ran out of food and starved to death',
             'intro': 'do you want to (1) stop at the next fort, (2) hunt, \n'
                      'or (3) continue'
             },
    'win': {'conclusion': 'president james k. polk sends you his\n'
                          '      heartiest congratulations\n'
                          '           and wishes you a prosperous life ahead\n'
                          '                      at your new home',
            'intro': 'you finally arrived at oregon city\n'
                     ' after {objective.total_distance} long miles---hooray!!!!!'
            }
}



#!/usr/bin/python
from datetime import timedelta
from datetime import datetime
import random
from tnalagmes import TNaLaGmesConstruct
from math import ceil


class ProgressTracker(TNaLaGmesConstruct):
    """
    "total distance"
    "current mileage"
    "add mileage"
    "subtract mileage"
    "are you completed?"
    "current difficulty
    """
    def __init__(self):
        TNaLaGmesConstruct.__init__(self, "progress_tracker")
        self._total_distance = 2040
        self._difficulty_trigger = 950
        self.last_advance = 0
        self.last_turn_fraction = 0
        self.mileage = 0
        self.medium_difficulty = False
        self.hard_difficulty = False

    def register_core_intents(self):
        """
            "total distance"
            "current mileage"
            "add mileage"
            "subtract mileage"
            "are you completed?"
            "yes"
            "no"
        """
        self.register_keyword_intent("total_distance",
                                     required=["total", "distance"],
                                     optionals=["question"],
                                     handler=self.handle_total_distance)
        self.register_keyword_intent("current_mileage",
                                     required=["mileage"],
                                     optionals=["question", "current"],
                                     handler=self.handle_mileage)
        self.register_keyword_intent("sub_mileage",
                                     required=["mileage", "subtract"],
                                     optionals=["question"],
                                     handler=self.handle_sub_mileage)
        self.register_keyword_intent("add_mileage",
                                     required=["mileage", "add"],
                                     optionals=["question"],
                                     handler=self.handle_add_mileage)
        self.register_keyword_intent("completed",
                                     required=["finished", "question"],
                                     optionals=["you"],
                                     handler=self.handle_completed)

    def handle_completed(self, intent=None):
        return "yes" if self.completed else "no"

    def handle_add_mileage(self, intent=None):
        amount = self.extract_number(intent["utterance"])
        self.add_mileage(amount)
        return "mileage increased by " + str(amount)

    def handle_sub_mileage(self, intent=None):
        amount = self.extract_number(intent["utterance"])
        self.subtract_mileage(amount)
        return "mileage diminished by " + str(amount)

    def handle_mileage(self, intent=None):
        return str(self.mileage)

    def handle_total_distance(self, intent=None):
        return str(self.total_distance)

    def random_advance(self, seed=150):
        # original logic from oregon trail
        self.last_advance = int(200 + (seed - 220) / 5 + 10 * random.random())
        self.mileage += self.last_advance

    # this function should only be used for increases in mileage
    # during a turn
    def add_mileage(self, gained_ground):
        self.last_advance += gained_ground
        self.mileage += int(gained_ground)

    def subtract_mileage(self, lost_ground):
        self.mileage -= int(lost_ground)
        if self.mileage < 0:
            self.mileage = 0

    def print_mileage(self):
        print("TOTAL MILEAGE IS", self.mileage)

    def difficulty_triggered(self):
        if self.mileage >= self._difficulty_trigger:
            return True
        return False

    @property
    def completed(self):
        if self.mileage >= self._total_distance:
            try:
                self.last_turn_fraction = (self._total_distance - self.last_advance) / (
                        self.mileage - self.last_advance)
            except ZeroDivisionError:
                self.last_turn_fraction = 0
            return True
        return False

    @property
    def total_distance(self):
        return self._total_distance

    @property
    def difficulty_threshold(self):
        return self._difficulty_trigger


class Calendar(TNaLaGmesConstruct):
    """
    Calendar Object

    "What day/date/week/weekday/month/year is it?"

    "next turn"
    "how many turns left"
    "how many turns passed"
    "how many days per turn"
    "maximum number of turns"

    "increase speed"
    "decrease speed"
    "rollback X days/months/weeks/years"
    "advance X days/months/weeks/years"

    """

    # standard python functionality, implementing date and turn tracking for our calendar
    def __init__(self, total_turns=20, start_date=None, turn_delta=0):
        TNaLaGmesConstruct.__init__(self, "calendar")  # pass a string with object_type if desired
        self._date = start_date or datetime.now()
        self.days_per_turn = turn_delta
        self._turn_delta = timedelta(days=self.days_per_turn) if self.days_per_turn else None
        self._turn_count = 1
        self._max_turns = total_turns

    def change_speed(self, days_per_turn=0):
        if isinstance(days_per_turn, str):
            if days_per_turn.strip().lower().startswith("easy"):
                days_per_turn = 1
            elif days_per_turn.strip().lower().startswith("hard"):
                days_per_turn = 14
            days_per_turn = self.extract_number(days_per_turn)

        self._turn_delta = timedelta(days_per_turn)

    def advance_date(self):
        if self._turn_delta is None:
            self._date = datetime.now()
            return
        self._date += self._turn_delta
        self._turn_count += 1

    def rollback_date(self, rollback_days=None):
        rollback_days = rollback_days or self.days_per_turn
        self._date -= timedelta(days=rollback_days)

    @property
    def weekday(self):
        day = self._date.weekday()
        if day == 0:
            return "Monday"
        elif day == 1:
            return "Tuesday"
        elif day == 2:
            return "Wednesday"
        elif day == 3:
            return "Thursday"
        elif day == 4:
            return "Friday"
        elif day == 5:
            return "Saturday"
        elif day == 6:
            return "Sunday"
        return "What kind of calendar is this? "

    @property
    def pretty_date(self):
        return self.weekday + " " + self._date.strftime('%d, %b %Y')

    @property
    def is_final_turn(self):
        if not self._turn_delta:
            return False
        if self._turn_count >= self._max_turns:
            return True
        else:
            return False

    @property
    def turns(self):
        return self._turn_count

    @property
    def date(self):
        return self._date

    @property
    def max_turns(self):
        return self._max_turns

    @property
    def week_of_month(self):
        # https://en.wikipedia.org/wiki/ISO_week_date#Weeks_per_month
        first_day = self.date.replace(day=1)

        day_of_month = self.date.day

        # adjust for sundays
        if (first_day.weekday() == 6):
            adjusted_dom = (1 + first_day.weekday()) / 7
        else:
            adjusted_dom = day_of_month + first_day.weekday()

        # days may belong to a month but their week may belong to another one
        # weeks belong to a month if its thursday falls on it
        if (first_day.weekday() > 3):
            return int(ceil(adjusted_dom / 7.0)) - 1
        else:
            return int(ceil(adjusted_dom / 7.0))

    # in here register the word triggers
    # im only using keyword rules instead of sample phrases
    # literal strings are used as keywords
    # if available on locale folder vocabulary is expanded
    def register_core_intents(self):
        self.register_keyword_intent("advance",
                                     required=["date", "advance"],
                                     optionals=[],
                                     handler=self.handle_advance)
        self.register_keyword_intent("rollback",
                                     required=["rollback", "date"],
                                     optionals=[],
                                     handler=self.handle_rollback)
        self.register_keyword_intent("speed_decrease",
                                     required=["decrease", "speed"],
                                     optionals=[],
                                     handler=self.handle_speed_decrease)
        self.register_keyword_intent("speed_increase",
                                     required=["increase", "speed"],
                                     optionals=[],
                                     handler=self.handle_speed_increase)
        self.register_keyword_intent("days_per_turn",
                                     required=["turns", "day"],
                                     optionals=["question"],
                                     handler=self.handle_days_per_turn)
        self.register_keyword_intent("turns_max",
                                     required=["turns", "total"],
                                     optionals=["question"],
                                     handler=self.handle_turns_max)
        self.register_keyword_intent("turns_past",
                                     required=["turns", "past"],
                                     optionals=["question"],
                                     handler=self.handle_turns_past)
        self.register_keyword_intent("turns_current",
                                     required=["turns", "current"],
                                     optionals=["question"],
                                     handler=self.handle_turns_current)
        self.register_keyword_intent("turns_left",
                                     required=["turns", "remaining"],
                                     optionals=["question"],
                                     handler=self.handle_turns_left)
        self.register_keyword_intent("next_turn",
                                     required=["next", "turn"],
                                     optionals=[],
                                     handler=self.handle_next_turn)
        self.register_keyword_intent("ask_day",
                                     required=["day"],
                                     optionals=["question"],
                                     handler=self.handle_day)
        self.register_keyword_intent("ask_week",
                                     required=["week"],
                                     optionals=["question"],
                                     handler=self.handle_week)
        self.register_keyword_intent("ask_weekday",
                                     required=["weekday"],
                                     optionals=["question"],
                                     handler=self.handle_weekday)
        self.register_keyword_intent("ask_month",
                                     required=["month"],
                                     optionals=["question"],
                                     handler=self.handle_month)
        self.register_keyword_intent("ask_year",
                                     required=["year"],
                                     optionals=["question"],
                                     handler=self.handle_year)
        self.register_keyword_intent("ask_date",
                                     required=["date"],
                                     optionals=["question"],
                                     handler=self.handle_date)

    # handlers for each natural language questions
    def handle_date(self, intent):
        return self.pretty_date

    def handle_week(self, intent):
        # TODO util to pronounce cardinals
        return "it is the " + str(self.week_of_month) + " week of the month"

    def handle_weekday(self, intent):
        return self.weekday

    def handle_month(self, intent):
        return str(self.date.month)

    def handle_day(self, intent):
        return str(self.date.day)

    def handle_year(self, intent):
        return "it is the year " + str(self.date.year)

    def handle_days_per_turn(self, intent):
        return str(self._turn_delta) + " days per turn"

    def handle_turns_left(self, intent):
        return "you have " + str(self.max_turns - self.turns) + " left"

    def handle_turns_max(self, intent):
        return "maximum number of turns is " + str(self.max_turns)

    def handle_turns_current(self, intent):
        return "currently in turn" + str(self.turns)

    def handle_turns_past(self, intent):
        return str(self.turns - 1) + " passed"

    def handle_next_turn(self, intent):
        self.advance_date()
        return "now in turn" + str(self.turns)

    def handle_advance(self, intent):
        self.advance_date()
        return "advanced date by " + str(self._turn_delta) + " days"

    def handle_rollback(self, intent):
        number = self.extract_number(intent["utterance"])
        self.rollback_date(number)
        return "rolled back date by " + str(number) + " days"

    def handle_speed_decrease(self, intent):
        number = self.extract_number(intent["utterance"])
        number = number or 1
        self.change_speed(self._turn_delta - number)
        return "decreased speed by " + str(number) + " days"

    def handle_speed_increase(self, intent):
        number = self.extract_number(intent["utterance"])
        number = number or 1
        self.change_speed(self._turn_delta - number)
        return "increased speed by " + str(number) + " days"


class InventoryItem(TNaLaGmesConstruct):
    """
    what are you
    what can you do
    how much are you worth
    """
    def __init__(self, name="thing", description="a thing", item_type="object"):
        self._value = 0
        self.name = name
        self.description = description
        self.item_type = item_type
        TNaLaGmesConstruct.__init__(self, name)


    def register_core_intents(self):
        "" """
            what are you
            what can you do
            how much are you worth
        """
        return


        self.register_keyword_intent("_name",
                                     required="_name",
                                     optionals=["question", "you"],
                                     handler=self.handle_total_distance)
        self.register_keyword_intent("describe",
                                     required=self.name,
                                     optionals=["question", "you"],
                                     handler=self.handle_total_distance)

        self.register_keyword_intent("value",
                                     required=["worth"],
                                     optionals=["question", "you"],
                                     handler=self.handle_mileage)
        self.register_keyword_intent("abilities",
                                     required=["abilities"],
                                     optionals=["question", "you"],
                                     handler=self.handle_mileage)

    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, value):
        self._value = int(value)

    def add(self, value):
        self._value += int(value)

    def subtract(self, value):
        self._value -= int(value)


class Inventory(TNaLaGmesConstruct):
    """
    what do you have
    how much money do you have
    how much are you worth
    """
    def __init__(self, start_money=700):
        self.start_money = start_money
        self._money = start_money
        self.items = []
        TNaLaGmesConstruct.__init__(self, "inventory")

    def register_core_intents(self):
        "" """
             what do you have
            how much money do you have
            how much are you worth
        """
        return

        self.register_keyword_intent("content",
                                     required="content",
                                     optionals=["question", "you"],
                                     handler=self.handle_total_distance)

        self.register_keyword_intent("money",
                                     required=["value"],
                                     optionals=["question", "you"],
                                     handler=self.handle_mileage)
        self.register_keyword_intent("value",
                                     required=["worth"],
                                     optionals=["question", "you"],
                                     handler=self.handle_mileage)

    @property
    def money(self):
        return self._money

    def spend(self, cost):
        self._money -= int(cost)

    def print_warnings(self):
        pass

    def print_inventory(self):
        pass


if __name__ == "__main__":
    construct = Calendar()
    print(construct.parse_command("current date"))
    print(construct.parse_command("what day"))
    print(construct.parse_command("what month"))
    print(construct.parse_command("what weekday"))
    print(construct.parse_command("total turns"))



#!/usr/bin/python
import random
from tnalagmes import TNaLaGmesConstruct


class Ability(TNaLaGmesConstruct):
    """
    what is your _name
    what is your cost
    what is your damage
    what are you
    generate some damage
    """
    def __init__(self, name, cost, dmg, type):
        TNaLaGmesConstruct.__init__(self, "ability")
        self.name = name
        self.cost = cost
        self.dmg = dmg
        self.type = type

    def register_core_intents(self):
        "" """
            what is your _name
            what is your cost
            what is your damage
            generate some damage
        """
        self.register_keyword_intent("_name",
                                     required="_name",
                                     optionals=["question", "you"],
                                     handler=self.handle_total_distance)

        self.register_keyword_intent("type",
                                     required=self.name,
                                     optionals=["question", "you"],
                                     handler=self.handle_total_distance)

        self.register_keyword_intent("damage",
                                     required=["damage"],
                                     optionals=["question", "you"],
                                     handler=self.handle_mileage)
        self.register_keyword_intent("create_damage",
                                     required=["damage", "create"],
                                     optionals=["you"],
                                     handler=self.handle_mileage)
        self.register_keyword_intent("cost",
                                     required=["value"],
                                     optionals=["question", "you"],
                                     handler=self.handle_mileage)

    def generate_damage(self):
        low = self.dmg - 15
        high = self.dmg + 15
        return random.randrange(low, high)





from time import sleep
import time
import random
from difflib import SequenceMatcher
from tnalagmes.util.time import now_local
from tnalagmes.util.log import LOG
from tnalagmes.util import resolve_resource_file
from pprint import pprint
from tnalagmes.lang.parse_en import *
from tnalagmes.lang.parse_es import *
from tnalagmes.lang.parse_fr import *
from tnalagmes.lang.parse_fr import *
from tnalagmes.lang.parse_fr import *
from tnalagmes.lang.parse_it import *
from tnalagmes.lang.parse_pt import *
from tnalagmes.lang.parse_sv import *
from os.path import expanduser, join, exists
from os import makedirs
import json


class TNaLaGmesConstruct(object):
    cache_dir = join(expanduser("~"), "tnalagmes", "intent_cache")

    if not exists(cache_dir):
        makedirs(cache_dir)

    def __init__(self, object_type="tnalagmes_object", direction=None,
                 interacting_with=None, scene=None, coordinates=None):
        """

        :param object_type: human string for object type identifier
        :param direction: direction object is facing 0 to 360 , where 0 is north
        :param interacting_with: where the object will send queries
        :param scene: where the object is located
        """

        from tnalagmes.intents import TNaLaGmesIntentContainer
        self.intent_parser = TNaLaGmesIntentContainer()
        self.object_type = object_type
        self.register_default_intents()
        self.register_core_intents()
        self.intent_parser.learn()

        self.waiting_for_user = False
        self._output = ""
        self.input = ""

        self.direction = direction
        self.interacting_with = interacting_with
        self.scene = scene
        self._coordinates = coordinates or [0]

    def move_to(self, coordinates):
        self._coordinates = coordinates

    def direction_to_int(self, direction):
        # handle front / left / right ?
        # handle north/south/west/...
        if direction == "up" or ("n" in direction and "w" not in direction and "e" not in direction):
            return 0  # north
        if direction == "down" or ("s" in direction and "w" not in direction and "e" not in direction):
            return 180  # south
        if direction == "right" or ("e" in direction and "s" not in direction and "n" not in direction):
            return 90  # east
        if direction == "left" or ("w" in direction and "s" not in direction and "n" not in direction):
            return 270  # west
        if "n" in direction and "w" in direction:
            return 315  # northwest
        if "s" in direction and "w" in direction:
            return 225  # southwest
        if "n" in direction and "e" in direction:
            return 45  # ne
        if "s" in direction and "e" in direction:
            return 135  # se

        # handle num degrees
        if "degrees" in direction:
            number = self.extract_number(direction)
            if number:
                # handle by 2 vs to 2
                if "by" in direction:
                    return self.direction + number
                return number
        return None # TODO raise exception?

    @staticmethod
    def fuzzy_match(x, against):
        """Perform a 'fuzzy' comparison between two strings.
        Returns:
            float: match percentage -- 1.0 for perfect match,
                   down to 0.0 for no match at all.
        """
        return SequenceMatcher(None, x, against).ratio()

    @staticmethod
    def match_one(query, choices):
        """
            Find best match from a list or dictionary given an input

            Arguments:
                query:   string to test
                choices: list or dictionary of choices

            Returns: tuple with best match, score
        """
        if isinstance(choices, dict):
            _choices = list(choices.keys())
        elif isinstance(choices, list):
            _choices = choices
        else:
            raise ValueError('a list or dict of choices must be provided')

        best = (_choices[0], TNaLaGmesConstruct.fuzzy_match(query, _choices[0]))
        for c in _choices[1:]:
            score = TNaLaGmesConstruct.fuzzy_match(query, c)
            if score > best[1]:
                best = (c, score)

        if isinstance(choices, dict):
            return choices[best[0]], best[1]
        else:
            return best

    @staticmethod
    def extract_number(text, short_scale=True, ordinals=False, lang="en-us"):
        """Takes in a string and extracts a number.

        Args:
            text (str): the string to extract a number from
            short_scale (bool): Use "short scale" or "long scale" for large
                numbers -- over a million.  The default is short scale, which
                is now common in most English speaking countries.
                See https://en.wikipedia.org/wiki/Names_of_large_numbers
            ordinals (bool): consider ordinal numbers, e.g. third=3 instead of 1/3
            lang (str): the BCP-47 code for the language to use
        Returns:
            (int, float or False): The number extracted or False if the input
                                   text contains no numbers
        """
        lang_lower = str(lang).lower()
        if lang_lower.startswith("en"):
            return extract_number_en(text, short_scale=short_scale,
                                     ordinals=ordinals)
        elif lang_lower.startswith("pt"):
            return extractnumber_pt(text)
        elif lang_lower.startswith("it"):
            return extractnumber_it(text)
        elif lang_lower.startswith("fr"):
            return extractnumber_fr(text)
        elif lang_lower.startswith("sv"):
            return extractnumber_sv(text)
        # elif lang_lower.startswith("de"):
        #    return extractnumber_de(text)
        # TODO: extractnumber_xx for other languages
        return text

    @staticmethod
    def extract_datetime(text, anchor=None, lang="en-us"):
        """
        Extracts date and time information from a sentence.  Parses many of the
        common ways that humans express dates and times, including relative dates
        like "5 days from today", "tomorrow', and "Tuesday".

        Vague terminology are given arbitrary values, like:
            - morning = 8 AM
            - afternoon = 3 PM
            - evening = 7 PM

        If a time isn't supplied or implied, the function defaults to 12 AM

        Args:
            text (str): the text to be interpreted
            anchor (:obj:`datetime`, optional): the date to be used for
                relative dating (for example, what does "tomorrow" mean?).
                Defaults to the current local date/time.
            lang (string): the BCP-47 code for the language to use

        Returns:
            [:obj:`datetime`, :obj:`str`]: 'datetime' is the extracted date
                as a datetime object in the user's local timezone.
                'leftover_string' is the original phrase with all date and time
                related keywords stripped out. See examples for further
                clarification

                Returns 'None' if the input string is empty.

        Examples:

            extract_datetime(
            ... "What is the weather like the day after tomorrow?",
            ... datetime(2017, 06, 30, 00, 00)
            ... )
            [datetime.datetime(2017, 7, 2, 0, 0), 'what is weather like']

            extract_datetime(
            ... "Set up an appointment 2 weeks from Sunday at 5 pm",
            ... datetime(2016, 02, 19, 00, 00)
            ... )
            [datetime.datetime(2016, 3, 6, 17, 0), 'set up appointment']
        """

        lang_lower = str(lang).lower()

        if not anchor:
            anchor = now_local()

        if lang_lower.startswith("en"):
            return extract_datetime_en(text, anchor)
        elif lang_lower.startswith("pt"):
            return extract_datetime_pt(text, anchor)
        elif lang_lower.startswith("it"):
            return extract_datetime_it(text, anchor)
        elif lang_lower.startswith("fr"):
            return extract_datetime_fr(text, anchor)
        elif lang_lower.startswith("sv"):
            return extract_datetime_sv(text, anchor)
        # TODO: extract_datetime for other languages
        return text

    @staticmethod
    def normalize(text, lang="en-us", remove_articles=True):
        """Prepare a string for parsing

        This function prepares the given text for parsing by making
        numbers consistent, getting rid of contractions, etc.
        Args:
            text (str): the string to normalize
            lang (str): the code for the language text is in
            remove_articles (bool): whether to remove articles (like 'a', or
                                    'the'). True by default.
        Returns:
            (str): The normalized string.
        """

        lang_lower = str(lang).lower()
        if lang_lower.startswith("en"):
            return normalize_en(text, remove_articles)
        elif lang_lower.startswith("es"):
            return normalize_es(text, remove_articles)
        elif lang_lower.startswith("pt"):
            return normalize_pt(text, remove_articles)
        elif lang_lower.startswith("it"):
            return normalize_it(text, remove_articles)
        elif lang_lower.startswith("fr"):
            return normalize_fr(text, remove_articles)
        elif lang_lower.startswith("sv"):
            return normalize_sv(text, remove_articles)
        # TODO: Normalization for other languages
        return text

    @staticmethod
    def word_gender(word, input_string="", lang="en-us"):
        '''
        guess gender of word, optionally use raw input text for context
        returns "m" if the word is male, "f" if female, False if unknown
        '''
        if "pt" in lang or "es" in lang:
            # spanish follows same rules
            return get_gender_pt(word, input_string)
        elif "it" in lang:
            return get_gender_it(word, input_string)

        return False

    @property
    def output(self):
        out = self._output
        self._output = ""
        return out

    def manual_fix_parse(self, text):
        # TODO replace vars
        return text

    @output.setter
    def output(self, text=""):
        if isinstance(text, list):
            text = [t.strip() for t in text if t.strip()]
            text = random.choice(text)
        else:
            if text is None or not text.strip():
                return
        self._output += self.manual_fix_parse(text) + "\n"

    def ask_yes_no(self, prompt):
        self.output = prompt
        self.waiting_for_user = True
        while self.waiting_for_user:
            sleep(0.1)
        response = self.normalize(self.input)
        if response[0] == 'y':
            return True
        if response[0] == 'n':
            return False
        else:
            return self.ask_yes_no(prompt)

    def ask_numeric(self, prompt, lower_bound=None, upper_bound=None):
        self.output = prompt
        self.waiting_for_user = True
        while self.waiting_for_user:
            sleep(0.1)
        response = self.extract_number(self.input)
        try:
            value = int(response)
        except ValueError:
            self.output = "impossible!"
            return self.ask_numeric(prompt, lower_bound, upper_bound)
        if lower_bound is not None:
            if value < lower_bound:
                self.output = str(response) + " is too low"
                return self.ask_numeric(prompt, lower_bound, upper_bound)
        if upper_bound is not None:
            if value > upper_bound:
                self.output = str(response) + " is too high"
                return self.ask_numeric(prompt, lower_bound, upper_bound)
        return value

    def ask_with_timeout(self, prompt="say BANG", answer="bang", timeout=7):
        self.output = prompt
        self.waiting_for_user = True
        while self.waiting_for_user:
            sleep(0.1)
        response = self.input.lower().strip()
        if response != answer:
            return self.ask_with_timeout(prompt, answer, timeout) + 1
        # TODO measure mic level or type speed
        return response, random.randint(1, 7)

    def register_default_intents(self):
        pass

    def register_core_intents(self):
        pass

    def calc_intents(self, utterance, lang="en-us"):
        return self.intent_parser.calc_intent(utterance)

    def register_intent(self, name, samples, handler=None):
        self.intent_parser.register_intent(name, samples)

    @staticmethod
    def load_resource(name, sep="\n", is_json=False):
        path = resolve_resource_file(name)
        if path and exists(path):
            with open(path, "r") as f:
                lines = f.read()
                if is_json:
                    return json.loads(lines)
                return lines.split(sep)
        return None

    def register_keyword_intent(self, name, required=None, optionals=None, handler=None, ignore_default_kw=False):
        self.intent_parser.register_keyword_intent(name, required, optionals, handler, ignore_default_kw)

    def parse_command(self, utterance):
        # parse intent
        intents = self.calc_intents(utterance)
        text = ""
        for intent in intents:
            text += self.intent_parser.execute_intent(intent)
        return text or "?"



if __name__ == "__main__":
    construct = TNaLaGmesConstruct()

    print(construct.parse_command("no"))
    print(construct.parse_command("yes"))
    print(construct.parse_command("what are you"))

    #print(construct.calc_intent("yes"))
    #print(construct.calc_intent("no"))
    #print(construct.calc_intent("what can you do"))

    # read buffer
    #print(construct.output) # empty
    # put something into the buffer
    #construct.output = "hello world"
    # read current buffer
    #print(construct.output)  # hello world
    #print(construct.output) # empty
    # put something into the buffer
    #construct.output = "hello world!"
    #construct.output = "how are you?"
    # read current buffer
    #print(construct.output)  # hello world\nhow are you?   #\n was added
    #print(construct.output)  # empty



from setuptools import setup

setup(
    name='TNaLaGmes',
    version='0.1.2',
    packages=['tnalagmes', 'tnalagmes.data', 'tnalagmes.games', 'tnalagmes.engines',
              'tnalagmes.models', 'tnalagmes.lang', 'tnalagmes.locale', 'tnalagmes.util'],
    url='https://github.com/JarbasAl/TNaLaGmes',
    license='Apache2.0',
    author='jarbasAI',
    author_email='jarbasai@mailfence.com',
    description='TNaLaGmes is a Toolbox for Natural Language Games'
)



import requests
from tnalagmes.util.log import LOG
ALLENNLP_URL = "http://demo.allennlp.org/predict/"

# TODO check for is_connected ?

# list of words that if present, trigger coref step
PRONOUNS_EN = ["i", "we", "me", "us", "you", "they", "them", "she", "he", "it", "her", "him",
               "that", "which", "who", "whom", "whose", "whichever", "whoever", "whomever",
               "this", "these", "those", "myself", "ourselves", "yourself", "yourselves",
               "himself", "herself", "itself", "themselves"]


def NER(text):
    ents = []
    # TODO on device
    try:
       data = {"model": "en_core_web_lg", "text": text}
       r = requests.post("https://api.explosion.ai/displacy/ent", data)
       r = r.json()
       for e in r:
           txt = text[e["start"]:e["end"]]
           ents.append((e["label"].lower(), txt))
    except Exception as e:
       LOG.error(e)
    return ents


def replace_coreferences(text):
    # TODO on device
    # "My sister has a dog. She loves him." -> "My sister has a dog. My sister loves a dog."
    for w in text.split(" "):
        # do not trigger coref resolution step if it isn't needed
        if w.lower() in PRONOUNS_EN:
            params = {"text": text}
            try:
                text = requests.get("https://coref.huggingface.co/coref", params=params).json()["corefResText"]
            except Exception as e:
                LOG.error(e)
            break
    return text


def textual_entailment(premise, hypothesis):
    # use the source: https://github.com/allenai/
    """
    Textual Entailment (TE) takes a pair of sentences and predicts whether the facts in the first necessarily imply the facts in the second one.
    The AllenNLP toolkit provides the following TE visualization, which can be run for any TE model you develop.
    This page demonstrates a reimplementation of the decomposable attention model (Parikh et al, 2017) ,
    which was state of the art for the SNLI benchmark (short sentences about visual scenes) in 2016.
    Rather than pre-trained Glove vectors, this model uses ELMo embeddings,
    which are completely character based and improve performance by 2%

    :param premise:
    :param hypotheses:
    :return:
    """
    url = ALLENNLP_URL + "textual-entailment"
    data = {"premise": premise,
            "hypothesis": hypothesis}
    r = requests.post(url, json=data).json()
    probs = r["label_probs"]
    return {"entailment": probs[0], "contradiction": probs[1], "neutral": probs[2]}

    #p = "If you help the needy, God will reward you."
    #h = "Giving money to the poor has good consequences."
    #print(textual_entailment_demo(p, h)) # {'contradiction': 0.04034089669585228, 'neutral': 0.1409262865781784, 'entailment': 0.8187329173088074}



# -*- coding: utf-8 -*-
#
# Copyright 2017 Mycroft AI Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from datetime import datetime

from dateutil.relativedelta import relativedelta

from tnalagmes.lang.parse_common import is_numeric, look_for_fractions


def extractnumber_sv(text):
    """
    This function prepares the given text for parsing by making
    numbers consistent, getting rid of contractions, etc.
    Args:
        text (str): the string to normalize
    Returns:
        (int) or (float): The value of extracted number
    """
    aWords = text.split()
    and_pass = False
    valPreAnd = False
    val = False
    count = 0
    while count < len(aWords):
        word = aWords[count]
        if is_numeric(word):
            val = float(word)
        elif word == "första":
            val = 1
        elif word == "andra":
            val = 2
        elif word == "tredje":
            val = 3
        elif word == "fjärde":
            val = 4
        elif word == "femte":
            val = 5
        elif word == "sjätte":
            val = 6
        elif is_fractional_sv(word):
            val = is_fractional_sv(word)
        else:
            if word == "en":
                val = 1
            if word == "ett":
                val = 1
            elif word == "två":
                val = 2
            elif word == "tre":
                val = 3
            elif word == "fyra":
                val = 4
            elif word == "fem":
                val = 5
            elif word == "sex":
                val = 6
            elif word == "sju":
                val = 7
            elif word == "åtta":
                val = 8
            elif word == "nio":
                val = 9
            elif word == "tio":
                val = 10
            if val:
                if count < (len(aWords) - 1):
                    wordNext = aWords[count + 1]
                else:
                    wordNext = ""
                valNext = is_fractional_sv(wordNext)

                if valNext:
                    val = val * valNext
                    aWords[count + 1] = ""

        if not val:
            # look for fractions like "2/3"
            aPieces = word.split('/')
            if look_for_fractions(aPieces):
                val = float(aPieces[0]) / float(aPieces[1])
            elif and_pass:
                # added to value, quit here
                val = valPreAnd
                break
            else:
                count += 1
                continue

        aWords[count] = ""

        if and_pass:
            aWords[count - 1] = ''  # remove "och"
            val += valPreAnd
        elif count + 1 < len(aWords) and aWords[count + 1] == 'och':
            and_pass = True
            valPreAnd = val
            val = False
            count += 2
            continue
        elif count + 2 < len(aWords) and aWords[count + 2] == 'och':
            and_pass = True
            valPreAnd = val
            val = False
            count += 3
            continue

        break

    if not val:
        return False

    return val


def extract_datetime_sv(string, currentDate):
    def clean_string(s):
        """
            cleans the input string of unneeded punctuation and capitalization
            among other things.
        """
        s = s.lower().replace('?', '').replace('.', '').replace(',', '') \
            .replace(' den ', ' ').replace(' en ', ' ')
        wordList = s.split()
        for idx, word in enumerate(wordList):
            word = word.replace("'s", "")

            ordinals = ["rd", "st", "nd", "th"]
            if word[0].isdigit():
                for ordinal in ordinals:
                    if ordinal in word:
                        word = word.replace(ordinal, "")
            wordList[idx] = word

        return wordList

    def date_found():
        return found or \
               (
                       datestr != "" or timeStr != "" or
                       yearOffset != 0 or monthOffset != 0 or
                       dayOffset is True or hrOffset != 0 or
                       hrAbs != 0 or minOffset != 0 or
                       minAbs != 0 or secOffset != 0
               )

    if string == "" or not currentDate:
        return None

    found = False
    daySpecified = False
    dayOffset = False
    monthOffset = 0
    yearOffset = 0
    dateNow = currentDate
    today = dateNow.strftime("%w")
    currentYear = dateNow.strftime("%Y")
    fromFlag = False
    datestr = ""
    hasYear = False
    timeQualifier = ""

    timeQualifiersList = ['morgon', 'förmiddag', 'eftermiddag', 'kväll']
    markers = ['på', 'i', 'den här', 'kring', 'efter']
    days = ['måndag', 'tisdag', 'onsdag', 'torsdag',
            'fredag', 'lördag', 'söndag']
    months = ['januari', 'februari', 'mars', 'april', 'maj', 'juni',
              'juli', 'augusti', 'september', 'oktober', 'november',
              'december']
    monthsShort = ['jan', 'feb', 'mar', 'apr', 'may', 'june', 'july', 'aug',
                   'sept', 'oct', 'nov', 'dec']

    words = clean_string(string)

    for idx, word in enumerate(words):
        if word == "":
            continue
        wordPrevPrev = words[idx - 2] if idx > 1 else ""
        wordPrev = words[idx - 1] if idx > 0 else ""
        wordNext = words[idx + 1] if idx + 1 < len(words) else ""
        wordNextNext = words[idx + 2] if idx + 2 < len(words) else ""

        # this isn't in clean string because I don't want to save back to words
        word = word.rstrip('s')
        start = idx
        used = 0
        # save timequalifier for later
        if word in timeQualifiersList:
            timeQualifier = word
            # parse today, tomorrow, day after tomorrow
        elif word == "idag" and not fromFlag:
            dayOffset = 0
            used += 1
        elif word == "imorgon" and not fromFlag:
            dayOffset = 1
            used += 1
        elif word == "morgondagen" or word == "morgondagens" and not fromFlag:
            dayOffset = 1
            used += 1
        elif word == "övermorgon" and not fromFlag:
            dayOffset = 2
            used += 1
        # parse 5 days, 10 weeks, last week, next week
        elif word == "dag" or word == "dagar":
            if wordPrev[0].isdigit():
                dayOffset += int(wordPrev)
                start -= 1
                used = 2
        elif word == "vecka" or word == "veckor" and not fromFlag:
            if wordPrev[0].isdigit():
                dayOffset += int(wordPrev) * 7
                start -= 1
                used = 2
            elif wordPrev == "nästa":
                dayOffset = 7
                start -= 1
                used = 2
            elif wordPrev == "förra":
                dayOffset = -7
                start -= 1
                used = 2
                # parse 10 months, next month, last month
        elif word == "månad" and not fromFlag:
            if wordPrev[0].isdigit():
                monthOffset = int(wordPrev)
                start -= 1
                used = 2
            elif wordPrev == "nästa":
                monthOffset = 1
                start -= 1
                used = 2
            elif wordPrev == "förra":
                monthOffset = -1
                start -= 1
                used = 2
                # parse 5 years, next year, last year
        elif word == "år" and not fromFlag:
            if wordPrev[0].isdigit():
                yearOffset = int(wordPrev)
                start -= 1
                used = 2
            elif wordPrev == "nästa":
                yearOffset = 1
                start -= 1
                used = 2
            elif wordPrev == "förra":
                yearOffset = -1
                start -= 1
                used = 2
                # parse Monday, Tuesday, etc., and next Monday,
                # last Tuesday, etc.
        elif word in days and not fromFlag:
            d = days.index(word)
            dayOffset = (d + 1) - int(today)
            used = 1
            if dayOffset < 0:
                dayOffset += 7
            if wordPrev == "nästa":
                dayOffset += 7
                used += 1
                start -= 1
            elif wordPrev == "förra":
                dayOffset -= 7
                used += 1
                start -= 1
        # parse 15 of July, June 20th, Feb 18, 19 of February
        elif word in months or word in monthsShort and not fromFlag:
            try:
                m = months.index(word)
            except ValueError:
                m = monthsShort.index(word)
            used += 1
            datestr = months[m]
            if wordPrev and (wordPrev[0].isdigit() or
                             (wordPrev == "of" and wordPrevPrev[0].isdigit())):
                if wordPrev == "of" and wordPrevPrev[0].isdigit():
                    datestr += " " + words[idx - 2]
                    used += 1
                    start -= 1
                else:
                    datestr += " " + wordPrev
                start -= 1
                used += 1
                if wordNext and wordNext[0].isdigit():
                    datestr += " " + wordNext
                    used += 1
                    hasYear = True
                else:
                    hasYear = False

            elif wordNext and wordNext[0].isdigit():
                datestr += " " + wordNext
                used += 1
                if wordNextNext and wordNextNext[0].isdigit():
                    datestr += " " + wordNextNext
                    used += 1
                    hasYear = True
                else:
                    hasYear = False
        # parse 5 days from tomorrow, 10 weeks from next thursday,
        # 2 months from July
        validFollowups = days + months + monthsShort
        validFollowups.append("idag")
        validFollowups.append("imorgon")
        validFollowups.append("nästa")
        validFollowups.append("förra")
        validFollowups.append("nu")
        if (word == "från" or word == "efter") and wordNext in validFollowups:
            used = 2
            fromFlag = True
            if wordNext == "imorgon":
                dayOffset += 1
            elif wordNext in days:
                d = days.index(wordNext)
                tmpOffset = (d + 1) - int(today)
                used = 2
                if tmpOffset < 0:
                    tmpOffset += 7
                dayOffset += tmpOffset
            elif wordNextNext and wordNextNext in days:
                d = days.index(wordNextNext)
                tmpOffset = (d + 1) - int(today)
                used = 3
                if wordNext == "nästa":
                    tmpOffset += 7
                    used += 1
                    start -= 1
                elif wordNext == "förra":
                    tmpOffset -= 7
                    used += 1
                    start -= 1
                dayOffset += tmpOffset
        if used > 0:
            if start - 1 > 0 and words[start - 1] == "denna":
                start -= 1
                used += 1

            for i in range(0, used):
                words[i + start] = ""

            if start - 1 >= 0 and words[start - 1] in markers:
                words[start - 1] = ""
            found = True
            daySpecified = True

    # parse time
    timeStr = ""
    hrOffset = 0
    minOffset = 0
    secOffset = 0
    hrAbs = 0
    minAbs = 0

    for idx, word in enumerate(words):
        if word == "":
            continue

        wordPrevPrev = words[idx - 2] if idx > 1 else ""
        wordPrev = words[idx - 1] if idx > 0 else ""
        wordNext = words[idx + 1] if idx + 1 < len(words) else ""
        wordNextNext = words[idx + 2] if idx + 2 < len(words) else ""
        # parse noon, midnight, morning, afternoon, evening
        used = 0
        if word == "middag":
            hrAbs = 12
            used += 1
        elif word == "midnatt":
            hrAbs = 0
            used += 1
        elif word == "morgon":
            if hrAbs == 0:
                hrAbs = 8
            used += 1
        elif word == "förmiddag":
            if hrAbs == 0:
                hrAbs = 10
            used += 1
        elif word == "eftermiddag":
            if hrAbs == 0:
                hrAbs = 15
            used += 1
        elif word == "kväll":
            if hrAbs == 0:
                hrAbs = 19
            used += 1
            # parse half an hour, quarter hour
        elif wordPrev in markers or wordPrevPrev in markers:
            if word == "halvtimme" or word == "halvtimma":
                minOffset = 30
            elif word == "kvart":
                minOffset = 15
            elif word == "timme" or word == "timma":
                hrOffset = 1
            words[idx - 1] = ""
            used += 1
            hrAbs = -1
            minAbs = -1
            # parse 5:00 am, 12:00 p.m., etc
        elif word[0].isdigit():
            isTime = True
            strHH = ""
            strMM = ""
            remainder = ""
            if ':' in word:
                # parse colons
                # "3:00 in the morning"
                stage = 0
                length = len(word)
                for i in range(length):
                    if stage == 0:
                        if word[i].isdigit():
                            strHH += word[i]
                        elif word[i] == ":":
                            stage = 1
                        else:
                            stage = 2
                            i -= 1
                    elif stage == 1:
                        if word[i].isdigit():
                            strMM += word[i]
                        else:
                            stage = 2
                            i -= 1
                    elif stage == 2:
                        remainder = word[i:].replace(".", "")
                        break
                if remainder == "":
                    nextWord = wordNext.replace(".", "")
                    if nextWord == "am" or nextWord == "pm":
                        remainder = nextWord
                        used += 1
                    elif nextWord == "tonight":
                        remainder = "pm"
                        used += 1
                    elif wordNext == "in" and wordNextNext == "the" and \
                            words[idx + 3] == "morning":
                        remainder = "am"
                        used += 3
                    elif wordNext == "in" and wordNextNext == "the" and \
                            words[idx + 3] == "afternoon":
                        remainder = "pm"
                        used += 3
                    elif wordNext == "in" and wordNextNext == "the" and \
                            words[idx + 3] == "evening":
                        remainder = "pm"
                        used += 3
                    elif wordNext == "in" and wordNextNext == "morning":
                        remainder = "am"
                        used += 2
                    elif wordNext == "in" and wordNextNext == "afternoon":
                        remainder = "pm"
                        used += 2
                    elif wordNext == "in" and wordNextNext == "evening":
                        remainder = "pm"
                        used += 2
                    elif wordNext == "this" and wordNextNext == "morning":
                        remainder = "am"
                        used = 2
                    elif wordNext == "this" and wordNextNext == "afternoon":
                        remainder = "pm"
                        used = 2
                    elif wordNext == "this" and wordNextNext == "evening":
                        remainder = "pm"
                        used = 2
                    elif wordNext == "at" and wordNextNext == "night":
                        if strHH > 5:
                            remainder = "pm"
                        else:
                            remainder = "am"
                        used += 2
                    else:
                        if timeQualifier != "":
                            if strHH <= 12 and \
                                    (timeQualifier == "evening" or
                                     timeQualifier == "afternoon"):
                                strHH += 12
            else:
                # try to parse # s without colons
                # 5 hours, 10 minutes etc.
                length = len(word)
                strNum = ""
                remainder = ""
                for i in range(length):
                    if word[i].isdigit():
                        strNum += word[i]
                    else:
                        remainder += word[i]

                if remainder == "":
                    remainder = wordNext.replace(".", "").lstrip().rstrip()

                if (
                        remainder == "pm" or
                        wordNext == "pm" or
                        remainder == "p.m." or
                        wordNext == "p.m."):
                    strHH = strNum
                    remainder = "pm"
                    used = 1
                elif (
                        remainder == "am" or
                        wordNext == "am" or
                        remainder == "a.m." or
                        wordNext == "a.m."):
                    strHH = strNum
                    remainder = "am"
                    used = 1
                else:
                    if wordNext == "pm" or wordNext == "p.m.":
                        strHH = strNum
                        remainder = "pm"
                        used = 1
                    elif wordNext == "am" or wordNext == "a.m.":
                        strHH = strNum
                        remainder = "am"
                        used = 1
                    elif (
                            int(word) > 100 and
                            (
                                    wordPrev == "o" or
                                    wordPrev == "oh"
                            )):
                        # 0800 hours (pronounced oh-eight-hundred)
                        strHH = int(word) / 100
                        strMM = int(word) - strHH * 100
                        if wordNext == "hours":
                            used += 1
                    elif (
                            wordNext == "hours" and
                            word[0] != '0' and
                            (
                                    int(word) < 100 and
                                    int(word) > 2400
                            )):
                        # "in 3 hours"
                        hrOffset = int(word)
                        used = 2
                        isTime = False
                        hrAbs = -1
                        minAbs = -1

                    elif wordNext == "minutes":
                        # "in 10 minutes"
                        minOffset = int(word)
                        used = 2
                        isTime = False
                        hrAbs = -1
                        minAbs = -1
                    elif wordNext == "seconds":
                        # in 5 seconds
                        secOffset = int(word)
                        used = 2
                        isTime = False
                        hrAbs = -1
                        minAbs = -1
                    elif int(word) > 100:
                        strHH = int(word) / 100
                        strMM = int(word) - strHH * 100
                        if wordNext == "hours":
                            used += 1
                    elif wordNext[0].isdigit():
                        strHH = word
                        strMM = wordNext
                        used += 1
                        if wordNextNext == "hours":
                            used += 1
                    elif (
                            wordNext == "" or wordNext == "o'clock" or
                            (
                                    wordNext == "in" and
                                    (
                                            wordNextNext == "the" or
                                            wordNextNext == timeQualifier
                                    )
                            )):
                        strHH = word
                        strMM = 00
                        if wordNext == "o'clock":
                            used += 1
                        if wordNext == "in" or wordNextNext == "in":
                            used += (1 if wordNext == "in" else 2)
                            if (wordNextNext and
                                    wordNextNext in timeQualifier or
                                    (words[words.index(wordNextNext) + 1] and
                                     words[words.index(wordNextNext) + 1] in
                                     timeQualifier)):
                                if (wordNextNext == "afternoon" or
                                        (len(words) >
                                         words.index(wordNextNext) + 1 and
                                         words[words.index(
                                             wordNextNext) + 1] == "afternoon")):
                                    remainder = "pm"
                                if (wordNextNext == "evening" or
                                        (len(words) >
                                         (words.index(wordNextNext) + 1) and
                                         words[words.index(
                                             wordNextNext) + 1] == "evening")):
                                    remainder = "pm"
                                if (wordNextNext == "morning" or
                                        (len(words) >
                                         words.index(wordNextNext) + 1 and
                                         words[words.index(
                                             wordNextNext) + 1] == "morning")):
                                    remainder = "am"
                    else:
                        isTime = False

            strHH = int(strHH) if strHH else 0
            strMM = int(strMM) if strMM else 0
            strHH = strHH + 12 if remainder == "pm" and strHH < 12 else strHH
            strHH = strHH - 12 if remainder == "am" and strHH >= 12 else strHH
            if strHH > 24 or strMM > 59:
                isTime = False
                used = 0
            if isTime:
                hrAbs = strHH * 1
                minAbs = strMM * 1
                used += 1
        if used > 0:
            # removed parsed words from the sentence
            for i in range(used):
                words[idx + i] = ""

            if wordPrev == "o" or wordPrev == "oh":
                words[words.index(wordPrev)] = ""

            if wordPrev == "early":
                hrOffset = -1
                words[idx - 1] = ""
                idx -= 1
            elif wordPrev == "late":
                hrOffset = 1
                words[idx - 1] = ""
                idx -= 1
            if idx > 0 and wordPrev in markers:
                words[idx - 1] = ""
            if idx > 1 and wordPrevPrev in markers:
                words[idx - 2] = ""

            idx += used - 1
            found = True

    # check that we found a date
    if not date_found:
        return None

    if dayOffset is False:
        dayOffset = 0

    # perform date manipulation

    extractedDate = dateNow
    extractedDate = extractedDate.replace(microsecond=0,
                                          second=0,
                                          minute=0,
                                          hour=0)
    if datestr != "":
        temp = datetime.strptime(datestr, "%B %d")
        if not hasYear:
            temp = temp.replace(year=extractedDate.year)
            if extractedDate < temp:
                extractedDate = extractedDate.replace(year=int(currentYear),
                                                      month=int(
                                                          temp.strftime(
                                                              "%m")),
                                                      day=int(temp.strftime(
                                                          "%d")))
            else:
                extractedDate = extractedDate.replace(
                    year=int(currentYear) + 1,
                    month=int(temp.strftime("%m")),
                    day=int(temp.strftime("%d")))
        else:
            extractedDate = extractedDate.replace(
                year=int(temp.strftime("%Y")),
                month=int(temp.strftime("%m")),
                day=int(temp.strftime("%d")))

    if timeStr != "":
        temp = datetime(timeStr)
        extractedDate = extractedDate.replace(hour=temp.strftime("%H"),
                                              minute=temp.strftime("%M"),
                                              second=temp.strftime("%S"))

    if yearOffset != 0:
        extractedDate = extractedDate + relativedelta(years=yearOffset)
    if monthOffset != 0:
        extractedDate = extractedDate + relativedelta(months=monthOffset)
    if dayOffset != 0:
        extractedDate = extractedDate + relativedelta(days=dayOffset)
    if hrAbs != -1 and minAbs != -1:

        extractedDate = extractedDate + relativedelta(hours=hrAbs,
                                                      minutes=minAbs)
        if (hrAbs != 0 or minAbs != 0) and datestr == "":
            if not daySpecified and dateNow > extractedDate:
                extractedDate = extractedDate + relativedelta(days=1)
    if hrOffset != 0:
        extractedDate = extractedDate + relativedelta(hours=hrOffset)
    if minOffset != 0:
        extractedDate = extractedDate + relativedelta(minutes=minOffset)
    if secOffset != 0:
        extractedDate = extractedDate + relativedelta(seconds=secOffset)
    for idx, word in enumerate(words):
        if words[idx] == "and" and words[idx - 1] == "" and words[
            idx + 1] == "":
            words[idx] = ""

    resultStr = " ".join(words)
    resultStr = ' '.join(resultStr.split())
    return [extractedDate, resultStr]


def is_fractional_sv(input_str):
    """
    This function takes the given text and checks if it is a fraction.

    Args:
        input_str (str): the string to check if fractional
    Returns:
        (bool) or (float): False if not a fraction, otherwise the fraction

    """
    if input_str.endswith('ars', -3):
        input_str = input_str[:len(input_str) - 3]  # e.g. "femtedelar"
    if input_str.endswith('ar', -2):
        input_str = input_str[:len(input_str) - 2]  # e.g. "femtedelar"
    if input_str.endswith('a', -1):
        input_str = input_str[:len(input_str) - 1]  # e.g. "halva"
    if input_str.endswith('s', -1):
        input_str = input_str[:len(input_str) - 1]  # e.g. "halva"

    aFrac = ["hel", "halv", "tredjedel", "fjärdedel", "femtedel", "sjättedel",
             "sjundedel", "åttondel", "niondel", "tiondel", "elftedel",
             "tolftedel"]
    if input_str.lower() in aFrac:
        return 1.0 / (aFrac.index(input_str) + 1)
    if input_str == "kvart":
        return 1.0 / 4
    if input_str == "trekvart":
        return 3.0 / 4

    return False


def normalize_sv(text, remove_articles):
    """ English string normalization """

    words = text.split()  # this also removed extra spaces
    normalized = ''
    for word in words:
        # Convert numbers into digits, e.g. "two" -> "2"
        if word == 'en':
            word = 'ett'
        textNumbers = ["noll", "ett", "två", "tre", "fyra", "fem", "sex",
                       "sju", "åtta", "nio", "tio", "elva", "tolv",
                       "tretton", "fjorton", "femton", "sexton",
                       "sjutton", "arton", "nitton", "tjugo"]
        if word in textNumbers:
            word = str(textNumbers.index(word))

        normalized += " " + word

    return normalized[1:]  # strip the initial space



TERMINOLOGY = {
    'ammunition': ['ammunition'],
    'armour': ['armour'],
    'attack_damage': ['injuries'],
    'currency': ['cash'],
    'enemy': ['strangers'],
    'fuel': ['fuel'],
    'medicine': ['medicine'],
    'passive_damage': ['disease'],
    'supplies': ['supplies']}

event_template = {
    "intro": "",
    "events": [],
    "conclusion": "",
    "error": "",
    "damage": "",
    "die": ""
}

RANDOM_EVENTS = {

    'attack': {'conclusion': 'enemies defeated',
               'damage': 'received damage',
               'die': '',
               'error': 'out of ammunition',
               'events': [],
               'intro': 'enemies attack'},
    'illness': {'conclusion': '',
                'damage': '',
                'die': '',
                'error': '',
                'events': [],
                'intro': ''},
    'poison': {'conclusion': '',
               'die': 'you die since you have no medicine',
               'error': '',
               'events': [],
               'intro': 'ate poisonous food'},
    'animal': {'conclusion': 'scared them away',
               'damage': 'got bitten',
               'die': '',
               'error': 'lost armour',
               'events': [],
               'intro': 'wild animals attack!'},
    'companion': {'conclusion': '',
                  'damage': '',
                  'die': '',
                  'error': '',
                  'events': [],
                  'intro': 'companion got injured'},

    'bad_terrain': {'conclusion': 'lose food and clothes',
                    'intro': 'bad terrain causes damage'},
    'companion_lose': {'conclusion': 'spend half the day looking for '
                                     'him',
                       'intro': 'your companion got lost'},
    'find_supplies': {'conclusion': '', 'intro': 'found supplies'},
    'fuel_damage': {'conclusion': 'spend time looking for it',
                    'intro': 'vehicle stolen'},
    'shelter_damage': {'conclusion': 'lose time and supplies fixing '
                                     'it',
                       'intro': 'shelter damage'},
    'shelter_fire': {'conclusion': 'supplies damaged',
                     'intro': 'shelter fire'},
    'supply_damage': {'conclusion': 'lose time looking for more',
                      'intro': 'supplies perished'},
    'vehicle_damage': {'conclusion': 'slows you down rest of trip',
                       'intro': 'vehicle_damage'}
    ,

    'rain': {'conclusion': 'time and supplies lost',
             'intro': 'rain'
             },
    'storm': {'conclusion': 'supplies damaged', 'intro': 'storm'},
    'fog': {'conclusion': 'time is lost',
            'intro': 'lose your way in heavy fog'},
    'cold': {'conclusion': 'you have enough clothing',
             'error': 'frostbite',
             'intro': 'cold'}
}

GAME_EVENTS = {
    'damage': {'conclusion': '',
               'error': 'you ran out medical supplies\nyou died of',
               'high': 'critical health --medicine used',
               'intro': '',
               'mild': 'low health ---medicine used'},
    'difficulty_damage': {'conclusion': '',
                          'error': '',
                          'events': [],
                          'intro': 'time and supplies lost'},
    'easy_difficulty': {'conclusion': 'you made it safely',
                        'events': ['you got lost---lose valuable time trying to '
                                   'find trail!',
                                   'vehicle damaged!---lose time and supplies',
                                   'dangerous path ahead']},
    'enemy_encounter': {'conclusion': '',
                        'damage': 'sustained damage',
                        'die': 'got killed by ',
                        'enemy_run': 'they did not attack',
                        'high_score': 'you drove them off',
                        'hostile_conclusion': ' were hostile',
                        'hostile_intro': 'they look hostile',
                        'intro': ' ahead.',
                        'low_score': 'kinda slow with your shooting',
                        'number_questions': ['(1) run  (2) attack  (3) continue  '
                                             '(4) circle'],
                        'peaceful_conclusion': ' were friendly',
                        'peaceful_intro': "they don't look hostile"},
    'explore': {'conclusion': '',
                'error': 'can not explore without ammunition',
                'events': ['found supplies!',
                           'sorry---no luck today',
                           'found a lot of supplies'],
                'intro': ''},
    'game_over': {'conclusion': 'distance remaining: ',
                  'error': 'time has run out',
                  'intro': ''},
    'hard_difficulty': {'conclusion': '', 'intro': ''},
    'heal': {'conclusion': "doctor's bill is $20",
             'die': 'you died of ',
             'error': "you can't afford a doctor",
             'events': [],
             'intro': ''},
    'intro': {'conclusion': 'try not to die', 'intro': 'travel to safety'},
    'inventory': {'conclusion': 'after all your purchases, you now have '
                                '{inv.money} dollars left',
                  'error': 'you overspent--you only had $ {inv.start_money} to '
                           'spend.  buy again',
                  'intro': 'how much do you want to spend on '},
    'lose': {'intro': 'you died', "conclusion": ""},
    'low_supplies': {'intro': 'low supplies'},
    'maintenance': {'conclusion': '',
                    'error': 'not enough supplies',
                    'intro': 'consume supplies (1) a little  (2) moderately (3) a lot'},
    'medium_difficulty': {'conclusion': 'you made it safely', 'intro': ''},
    'numeric': {'error': 'impossible', 'high': 'too high', 'low': 'too low'},
    'shop': {'conclusion': '',
             'error': "you don't have that much--keep your spending down",
             'events': [],
             'intro': 'enter what you wish to spend on the following'},
    'turn': {'die': 'you ran out of supplies and starved to death',
             'intro': '(1) shop, (2) explore, (3) travel'},
    'win': {'intro': 'you survived'}}



# -*- coding: utf-8 -*-
#
# Copyright 2017 Mycroft AI Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from tnalagmes.lang.format_common import convert_to_mixed_fraction

FRACTION_STRING_PT = {
    2: 'meio',
    3: u'terço',
    4: 'quarto',
    5: 'quinto',
    6: 'sexto',
    7: u'sétimo',
    8: 'oitavo',
    9: 'nono',
    10: u'décimo',
    11: 'onze avos',
    12: 'doze avos',
    13: 'treze avos',
    14: 'catorze avos',
    15: 'quinze avos',
    16: 'dezasseis avos',
    17: 'dezassete avos',
    18: 'dezoito avos',
    19: 'dezanove avos',
    20: u'vigésimo',
    30: u'trigésimo',
    100: u'centésimo',
    1000: u'milésimo'
}


def nice_number_pt(number, speech, denominators):
    """ Portuguese helper for nice_number

    This function formats a float to human understandable functions. Like
    4.5 becomes "4 e meio" for speech and "4 1/2" for text

    Args:
        number (int or float): the float to format
        speech (bool): format for speech (True) or display (False)
        denominators (iter of ints): denominators to use, default [1 .. 20]
    Returns:
        (str): The formatted string.
    """

    result = convert_to_mixed_fraction(number, denominators)
    if not result:
        # Give up, just represent as a 3 decimal number
        return str(round(number, 3))

    whole, num, den = result

    if not speech:
        if num == 0:
            # TODO: Number grouping?  E.g. "1,000,000"
            return str(whole)
        else:
            return '{} {}/{}'.format(whole, num, den)

    if num == 0:
        return str(whole)
    # denominador
    den_str = FRACTION_STRING_PT[den]
    # fracções
    if whole == 0:
        if num == 1:
            # um décimo
            return_string = 'um {}'.format(den_str)
        else:
            # três meio
            return_string = '{} {}'.format(num, den_str)
    # inteiros >10
    elif num == 1:
        # trinta e um
        return_string = '{} e {}'.format(whole, den_str)
    # inteiros >10 com fracções
    else:
        # vinte e 3 décimo
        return_string = '{} e {} {}'.format(whole, num, den_str)
    # plural
    if num > 1:
        return_string += 's'
    return return_string



from tnalagmes.data.oregon_trail_data import TERMINOLOGY, RANDOM_EVENTS, GAME_EVENTS
from tnalagmes.engines.oregon75 import Oregon75Engine


class Oregon78Engine(Oregon75Engine):
    """ so called because logic is ported from 1978 basic version of oregon trail"""
    # https://www.filfre.net/misc/oregon1978.bas

    TERMINOLOGY = TERMINOLOGY
    DATA = GAME_EVENTS
    RANDOM_EVENTS = RANDOM_EVENTS
    name = "Oregon78Engine"

    def __init__(self, name=None, from_json=False):
        Oregon75Engine.__init__(self, from_json)


if __name__ == "__main__":
    game = Oregon78Engine()
    game.run()



# -*- coding: utf-8 -*-
#
# Copyright 2017 Mycroft AI Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from tnalagmes.lang.format_common import convert_to_mixed_fraction
from math import floor

months = ['januar', 'februar', 'märz', 'april', 'mai', 'juni',
          'juli', 'august', 'september', 'oktober', 'november',
          'dezember']

NUM_STRING_DE = {
    0: 'null',
    1: 'ein',  # ein Viertel etc., nicht eins Viertel
    2: 'zwei',
    3: 'drei',
    4: 'vier',
    5: u'fünf',
    6: 'sechs',
    7: 'sieben',
    8: 'acht',
    9: 'neun',
    10: 'zehn',
    11: 'elf',
    12: u'zwölf',
    13: 'dreizehn',
    14: 'vierzehn',
    15: u'fünfzehn',
    16: 'sechzehn',
    17: 'siebzehn',
    18: 'achtzehn',
    19: 'neunzehn',
    20: 'zwanzig',
    30: u'dreißig',
    40: 'vierzig',
    50: u'fünfzig',
    60: 'sechzig',
    70: 'siebzig',
    80: 'achtzig',
    90: 'neunzig',
    100: 'hundert'
}

# German uses "long scale" https://en.wikipedia.org/wiki/Long_and_short_scales
# Currently, numbers are limited to 1000000000000000000000000,
# but NUM_POWERS_OF_TEN can be extended to include additional number words


NUM_POWERS_OF_TEN = [
    '', 'tausend', 'Million', 'Milliarde', 'Billion', 'Billiarde', 'Trillion',
    'Trilliarde'
]

FRACTION_STRING_DE = {
    2: 'halb',
    3: 'drittel',
    4: 'viertel',
    5: u'fünftel',
    6: 'sechstel',
    7: 'siebtel',
    8: 'achtel',
    9: 'neuntel',
    10: 'zehntel',
    11: 'elftel',
    12: u'zwölftel',
    13: 'dreizehntel',
    14: 'vierzehntel',
    15: u'fünfzehntel',
    16: 'sechzehntel',
    17: 'siebzehntel',
    18: 'achtzehntel',
    19: 'neunzehntel',
    20: 'zwanzigstel'
}

# Numbers below 1 million are written in one word in German, yielding very
# long words
# In some circumstances it may better to seperate individual words
# Set EXTRA_SPACE=" " for separating numbers below 1 million (
# orthographically incorrect)
# Set EXTRA_SPACE="" for correct spelling, this is standard

# EXTRA_SPACE = " "
EXTRA_SPACE = ""


def nice_number_de(number, speech, denominators):
    """ German helper for nice_number
    This function formats a float to human understandable functions. Like
    4.5 becomes "4 einhalb" for speech and "4 1/2" for text
    Args:
        number (int or float): the float to format
        speech (bool): format for speech (True) or display (False)
        denominators (iter of ints): denominators to use, default [1 .. 20]
    Returns:
        (str): The formatted string.
    """
    result = convert_to_mixed_fraction(number, denominators)
    if not result:
        # Give up, just represent as a 3 decimal number
        return str(round(number, 3)).replace(".", ",")
    whole, num, den = result
    if not speech:
        if num == 0:
            # TODO: Number grouping?  E.g. "1,000,000"
            return str(whole)
        else:
            return '{} {}/{}'.format(whole, num, den)
    if num == 0:
        return str(whole)
    den_str = FRACTION_STRING_DE[den]
    if whole == 0:
        if num == 1:
            return_string = 'ein {}'.format(den_str)
        else:
            return_string = '{} {}'.format(num, den_str)
    elif num == 1:
        return_string = '{} und ein {}'.format(whole, den_str)
    else:
        return_string = '{} und {} {}'.format(whole, num, den_str)

    return return_string


def pronounce_number_de(num, places=2):
    """
    Convert a number to its spoken equivalent
    For example, '5.2' would return 'five point two'
    Args:
        num(float or int): the number to pronounce (set limit below)
        places(int): maximum decimal places to speak
    Returns:
        (str): The pronounced number

    """

    def pronounce_triplet_de(num):
        result = ""
        num = floor(num)
        if num > 99:
            hundreds = floor(num / 100)
            if hundreds > 0:
                result += NUM_STRING_DE[
                              hundreds] + EXTRA_SPACE + 'hundert' + EXTRA_SPACE
                num -= hundreds * 100
        if num == 0:
            result += ''  # do nothing
        elif num == 1:
            result += 'eins'  # need the s for the last digit
        elif num <= 20:
            result += NUM_STRING_DE[num]  # + EXTRA_SPACE
        elif num > 20:
            ones = num % 10
            tens = num - ones
            if ones > 0:
                result += NUM_STRING_DE[ones] + EXTRA_SPACE
                if tens > 0:
                    result += 'und' + EXTRA_SPACE
            if tens > 0:
                result += NUM_STRING_DE[tens] + EXTRA_SPACE
        return result

    def pronounce_fractional_de(num,
                                places):  # fixed number of places even with
        # trailing zeros
        result = ""
        place = 10
        while places > 0:  # doesn't work with 1.0001 and places = 2: int(
            # num*place) % 10 > 0 and places > 0:
            result += " " + NUM_STRING_DE[int(num * place) % 10]
            if int(num * place) % 10 == 1:
                result += 's'  # "1" is pronounced "eins" after the decimal
                # point
            place *= 10
            places -= 1
        return result

    def pronounce_whole_number_de(num, scale_level=0):
        if num == 0:
            return ''

        num = floor(num)
        result = ''
        last_triplet = num % 1000

        if last_triplet == 1:
            if scale_level == 0:
                if result != '':
                    result += '' + 'eins'
                else:
                    result += "eins"
            elif scale_level == 1:
                result += 'ein' + EXTRA_SPACE + 'tausend' + EXTRA_SPACE
            else:
                result += "eine " + NUM_POWERS_OF_TEN[scale_level] + ' '
        elif last_triplet > 1:
            result += pronounce_triplet_de(last_triplet)
            if scale_level == 1:
                # result += EXTRA_SPACE
                result += 'tausend' + EXTRA_SPACE
            if scale_level >= 2:
                # if EXTRA_SPACE == '':
                #    result += " "
                result += " " + NUM_POWERS_OF_TEN[scale_level]
            if scale_level >= 2:
                if scale_level % 2 == 0:
                    result += "e"  # MillionE
                result += "n "  # MilliardeN, MillioneN

        num = floor(num / 1000)
        scale_level += 1
        return pronounce_whole_number_de(num,
                                         scale_level) + result  # + EXTRA_SPACE

    result = ""
    if abs(num) >= 1000000000000000000000000:  # cannot do more than this
        return str(num)
    elif num == 0:
        return str(NUM_STRING_DE[0])
    elif num < 0:
        return "minus " + pronounce_number_de(abs(num), places)
    else:
        if num == int(num):
            return pronounce_whole_number_de(num)
        else:
            whole_number_part = floor(num)
            fractional_part = num - whole_number_part
            result += pronounce_whole_number_de(whole_number_part)
            if places > 0:
                result += " Komma"
                result += pronounce_fractional_de(fractional_part, places)
            return result


def pronounce_ordinal_de(num):
    # ordinals for 1, 3, 7 and 8 are irregular
    # this produces the base form, it will have to be adapted for genus,
    # casus, numerus

    ordinals = ["nullte", "erste", "zweite", "dritte", "vierte", u"fünfte",
                "sechste", "siebte", "achte"]

    # only for whole positive numbers including zero
    if num < 0 or num != int(num):
        return num
    elif num < 9:
        return ordinals[num]
    elif num < 20:
        return pronounce_number_de(num) + "te"
    else:
        return pronounce_number_de(num) + "ste"


def nice_time_de(dt, speech=True, use_24hour=False, use_ampm=False):
    """
    Format a time to a comfortable human format

    For example, generate 'five thirty' for speech or '5:30' for
    text display.

    Args:
        dt (datetime): date to format (assumes already in local timezone)
        speech (bool): format for speech (default/True) or display (False)=Fal
        use_24hour (bool): output in 24-hour/military or 12-hour format
        use_ampm (bool): include the am/pm for 12-hour format
    Returns:
        (str): The formatted time string
    """
    if use_24hour:
        # e.g. "03:01" or "14:22"
        string = dt.strftime("%H:%M")
    else:
        if use_ampm:
            # e.g. "3:01 AM" or "2:22 PM"
            string = dt.strftime("%I:%M %p")
        else:
            # e.g. "3:01" or "2:22"
            string = dt.strftime("%I:%M")
        if string[0] == '0':
            string = string[1:]  # strip leading zeros

    if not speech:
        return string

    # Generate a speakable version of the time
    speak = ""
    if use_24hour:
        if dt.hour == 1:
            speak += "ein"  # 01:00 is "ein Uhr" not "eins Uhr"
        else:
            speak += pronounce_number_de(dt.hour)
        speak += " Uhr"
        if not dt.minute == 0:  # zero minutes are not pronounced, 13:00 is
            # "13 Uhr" not "13 hundred hours"
            speak += " " + pronounce_number_de(dt.minute)

        return speak  # ampm is ignored when use_24hour is true
    else:
        if dt.hour == 0 and dt.minute == 0:
            return "Mitternacht"
        if dt.hour == 12 and dt.minute == 0:
            return "Mittag"
        # TODO: "half past 3", "a quarter of 4" and other idiomatic times

        if dt.hour == 0:
            speak += pronounce_number_de(12)
        elif dt.hour <= 13:
            if dt.hour == 1 or dt.hour == 13:  # 01:00 and 13:00 is "ein Uhr"
                # not "eins Uhr"
                speak += 'ein'
            else:
                speak += pronounce_number_de(dt.hour)
        else:
            speak += pronounce_number_de(dt.hour - 12)

        speak += " Uhr"

        if not dt.minute == 0:
            speak += " " + pronounce_number_de(dt.minute)

        if use_ampm:
            if dt.hour > 11:
                if dt.hour < 18:
                    speak += " nachmittags"  # 12:01 - 17:59
                    # nachmittags/afternoon
                elif dt.hour < 22:
                    speak += " abends"  # 18:00 - 21:59 abends/evening
                else:
                    speak += " nachts"  # 22:00 - 23:59 nachts/at night
            elif dt.hour < 3:
                speak += " nachts"  # 00:01 - 02:59 nachts/at night
            else:
                speak += " morgens"  # 03:00 - 11:59 morgens/in the morning

        return speak


def nice_response_de(text):
    # check for months and call nice_ordinal_de declension of ordinals
    # replace "^" with "hoch" (to the power of)
    words = text.split()

    for idx, word in enumerate(words):
        if word.lower() in months:
            text = nice_ordinal_de(text)

        if word == '^':
            wordNext = words[idx + 1] if idx + 1 < len(words) else ""
            if wordNext.isnumeric():
                words[idx] = "hoch"
                text = " ".join(words)
    return text


def nice_ordinal_de(text):
    # check for months for declension of ordinals before months
    # depending on articles/prepositions
    normalized_text = text
    words = text.split()

    for idx, word in enumerate(words):
        wordNext = words[idx + 1] if idx + 1 < len(words) else ""
        wordPrev = words[idx - 1] if idx > 0 else ""
        if word[-1:] == ".":
            if word[:-1].isdecimal():
                if wordNext.lower() in months:
                    word = pronounce_ordinal_de(int(word[:-1]))
                    if wordPrev.lower() in ["am", "dem", "vom", "zum",
                                            "(vom", "(am", "zum"]:
                        word += "n"
                    elif wordPrev.lower() not in ["der", "die", "das"]:
                        word += "r"
                    words[idx] = word
            normalized_text = " ".join(words)
    return normalized_text



from mtranslate import translate as tx
from tnalagmes.util import play_mp3, play_wav
from os.path import expanduser
from os import system


GOOGLE_LANGUAGES = {
    'af': 'Afrikaans',
    'sq': 'Albanian',
    'ar': 'Arabic',
    'hy': 'Armenian',
    'bn': 'Bengali',
    'ca': 'Catalan',
    'zh': 'Chinese',
    'zh-cn': 'Chinese (Mandarin/China)',
    'zh-tw': 'Chinese (Mandarin/Taiwan)',
    'zh-yue': 'Chinese (Cantonese)',
    'hr': 'Croatian',
    'cs': 'Czech',
    'da': 'Danish',
    'nl': 'Dutch',
    'en': 'English',
    'en-au': 'English (Australia)',
    'en-uk': 'English (United Kingdom)',
    'en-us': 'English (United States)',
    'en-ca': 'English (Canada)',
    'en-gh': 'English (Ghana)',
    'en-in': 'English (India)',
    'en-ie': 'English (Ireland)',
    'en-nz': 'English (New Zealand)',
    'en-ng': 'English (Nigeria)',
    'en-ph': 'English (Philippines)',
    'en-za': 'English (South Africa)',
    'en-tz': 'English (Tanzania)',
    'eo': 'Esperanto',
    'fi': 'Finnish',
    'fr': 'French',
    'de': 'German',
    'el': 'Greek',
    'hi': 'Hindi',
    'hu': 'Hungarian',
    'is': 'Icelandic',
    'id': 'Indonesian',
    'it': 'Italian',
    'ja': 'Japanese',
    'km': 'Khmer (Cambodian)',
    'ko': 'Korean',
    'la': 'Latin',
    'lv': 'Latvian',
    'mk': 'Macedonian',
    'no': 'Norwegian',
    'pl': 'Polish',
    'pt': 'Portuguese',
    'pt-br': 'Brazilian Portuguese',
    'ro': 'Romanian',
    'ru': 'Russian',
    'sr': 'Serbian',
    'si': 'Sinhala',
    'sk': 'Slovak',
    'es': 'Spanish',
    'es-es': 'Spanish (Spain)',
    'es-us': 'Spanish (United States)',
    'sw': 'Swahili',
    'sv': 'Swedish',
    'ta': 'Tamil',
    'th': 'Thai',
    'tr': 'Turkish',
    'uk': 'Ukrainian',
    'vi': 'Vietnamese',
    'cy': 'Welsh'
}


def translate_text(text, lang="en-us"):
    sentence = tx(text, lang)
    return sentence


def say_in_language(sentence, lang="en-us", wav_file="~/tnalagmes/translated"):
    sentence = translate_text(sentence, lang)
    type = "mp3"
    wav_file += "." + type
    wav_file = expanduser(wav_file)
    get_sentence = 'wget -q -U Mozilla -O' + wav_file + \
                   ' "https://translate.google.com/translate_tts?tl=' + \
                   lang + '&q=' + sentence + '&client=tw-ob' + '"'
    system(get_sentence)
    if type == "mp3":
        play_mp3(wav_file)
    elif type == "wav":
        play_wav(wav_file)
    return sentence
